import json
from decimal import Decimal

import pytest
from model_bakery import baker

from fixtures.pbn_api import _zrob_wydawnictwo_pbn
from pbn_api.adapters.wydawnictwo import WydawnictwoPBNAdapter
from pbn_api.exceptions import PKZeroExportDisabled, WillNotExportError

from bpp import const
from bpp.models import (
    Autor,
    Czas_Udostepnienia_OpenAccess,
    Licencja_OpenAccess,
    Tryb_OpenAccess_Wydawnictwo_Ciagle,
    Wersja_Tekstu_OpenAccess,
    Wydawnictwo_Zwarte,
    Wydawnictwo_Zwarte_Autor,
)


def test_WydawnictwoPBNAdapter_ciagle(pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina):
    res = WydawnictwoPBNAdapter(
        pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina
    ).pbn_get_json()
    assert res["journal"]


def test_WydawnictwoPBNAdapter_nie_wysylaj_prac_bez_pk(
    pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina, pbn_uczelnia
):
    pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina.punkty_kbn = 0
    pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina.save()

    pbn_uczelnia.pbn_api_nie_wysylaj_prac_bez_pk = True
    pbn_uczelnia.save()

    with pytest.raises(PKZeroExportDisabled):
        WydawnictwoPBNAdapter(
            pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina, None, pbn_uczelnia
        ).pbn_get_json()


def test_WydawnictwoPBNAdapter_zwarte_ksiazka(pbn_wydawnictwo_zwarte_ksiazka):
    res = WydawnictwoPBNAdapter(pbn_wydawnictwo_zwarte_ksiazka).pbn_get_json()
    assert not res.get("journal")


def test_WydawnictwoPBNAdapter_zwarte_rozdzial(pbn_rozdzial_z_autorem_z_dyscyplina):
    res = WydawnictwoPBNAdapter(pbn_rozdzial_z_autorem_z_dyscyplina).pbn_get_json()
    assert not res.get("journal")


@pytest.fixture
def praca_z_dyscyplina_pbn(praca_z_dyscyplina, pbn_jezyk):
    _zrob_wydawnictwo_pbn(praca_z_dyscyplina, pbn_jezyk)
    return praca_z_dyscyplina


@pytest.fixture
def rozdzial_z_dyscyplina_pbn(praca_z_dyscyplina_pbn):
    cf = praca_z_dyscyplina_pbn.charakter_formalny
    cf.rodzaj_pbn = const.RODZAJ_PBN_ROZDZIAL
    cf.save()

    return praca_z_dyscyplina_pbn


def test_WydawnictwoPBNAdapter_autor_eksport(
    pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina, jednostka
):

    autor_bez_dyscypliny = baker.make(Autor)
    pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina.dodaj_autora(
        autor_bez_dyscypliny, jednostka, "Jan Budnik"
    )

    res = WydawnictwoPBNAdapter(
        pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina
    ).pbn_get_json()
    assert res["journal"]


@pytest.mark.django_db
def test_WydawnictwoPBNAdapter_pk_rowne_zero_eksport_wylaczony(
    pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina, jednostka, rf, pbn_uczelnia
):

    autor_bez_dyscypliny = baker.make(Autor)
    pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina.dodaj_autora(
        autor_bez_dyscypliny, jednostka, "Jan Budnik"
    )

    pbn_uczelnia.pbn_api_nie_wysylaj_prac_bez_pk = True
    pbn_uczelnia.save()

    with pytest.raises(PKZeroExportDisabled):
        WydawnictwoPBNAdapter(
            pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina, uczelnia=pbn_uczelnia
        ).pbn_get_json()


def test_WydawnictwoPBNAdapter_przypinanie_dyscyplin(
    pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina, jednostka
):
    wydawnictwo_autor: Wydawnictwo_Zwarte_Autor = (
        pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina.autorzy_set.first()
    )
    wydawnictwo_autor.przypieta = False
    wydawnictwo_autor.save()

    with pytest.raises(WillNotExportError, match="bez zadeklarowanych"):
        WydawnictwoPBNAdapter(
            pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina
        ).pbn_get_json()

    wydawnictwo_autor.przypieta = True
    wydawnictwo_autor.save()

    res = WydawnictwoPBNAdapter(
        pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina
    ).pbn_get_json()
    assert res["statements"]


def test_WydawnictwoPBNAdapter_eksport_artykulu_bez_oswiadczen_zwraca_blad(
    praca_z_dyscyplina_pbn,
):
    with pytest.raises(WillNotExportError, match="bez zadeklarowanych"):
        WydawnictwoPBNAdapter(praca_z_dyscyplina_pbn).pbn_get_json()


def test_WydawnictwoPBNAdapter_eksport_rozdzialu_bez_oswiadczen_zwraca_blad(
    rozdzial_z_dyscyplina_pbn,
):
    with pytest.raises(WillNotExportError, match="bez zadeklarowanych"):
        WydawnictwoPBNAdapter(rozdzial_z_dyscyplina_pbn).pbn_get_json()


@pytest.mark.django_db
def test_WydawnictwoPBNAdapter_www_eksport(
    pbn_rozdzial_z_autorem_z_dyscyplina, wydawnictwo_nadrzedne, pbn_jezyk, denorms
):
    wydawnictwo_zwarte = pbn_rozdzial_z_autorem_z_dyscyplina

    assert wydawnictwo_zwarte.wydawnictwo_nadrzedne_id == wydawnictwo_nadrzedne.pk

    _zrob_wydawnictwo_pbn(
        wydawnictwo_zwarte, pbn_jezyk, rodzaj_pbn=const.RODZAJ_PBN_ROZDZIAL
    )

    WWW = "https://www.example.com/"
    WWW2 = "https://www.example2.com/"

    wydawnictwo_nadrzedne.public_www = WWW
    wydawnictwo_nadrzedne.save()

    wydawnictwo_zwarte.www = wydawnictwo_zwarte.public_www = None
    wydawnictwo_zwarte.wydawnictwo_nadrzedne = wydawnictwo_nadrzedne
    wydawnictwo_zwarte.save()

    res = WydawnictwoPBNAdapter(wydawnictwo_zwarte).pbn_get_json()
    assert res["publicUri"] == WWW

    wydawnictwo_zwarte.www = WWW2
    wydawnictwo_zwarte.save()

    assert wydawnictwo_zwarte.wydawnictwo_nadrzedne == wydawnictwo_nadrzedne

    res = WydawnictwoPBNAdapter(wydawnictwo_zwarte).pbn_get_json()
    assert res["publicUri"] == WWW2


def test_WydawnictwoPBNAdapter_openaccess_zero_miesiecy_gdy_licencja(
    pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina: Wydawnictwo_Zwarte, openaccess_data
):

    praca_z_dyscyplina_pbn = pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina

    praca_z_dyscyplina_pbn.openaccess_licencja = Licencja_OpenAccess.objects.first()
    praca_z_dyscyplina_pbn.openaccess_tryb_dostepu = (
        Tryb_OpenAccess_Wydawnictwo_Ciagle.objects.first()
    )
    praca_z_dyscyplina_pbn.openaccess_wersja_tekstu = (
        Wersja_Tekstu_OpenAccess.objects.first()
    )
    praca_z_dyscyplina_pbn.openaccess_czas_publikacji = (
        Czas_Udostepnienia_OpenAccess.objects.get(nazwa="po opublikowaniu")
    )
    praca_z_dyscyplina_pbn.openaccess_ilosc_miesiecy = 12
    praca_z_dyscyplina_pbn.save()

    res = WydawnictwoPBNAdapter(praca_z_dyscyplina_pbn).pbn_get_json()
    assert res["openAccess"]["months"] == "12"

    praca_z_dyscyplina_pbn.openaccess_ilosc_miesiecy = None
    praca_z_dyscyplina_pbn.save()

    res = WydawnictwoPBNAdapter(praca_z_dyscyplina_pbn).pbn_get_json()
    assert res["openAccess"]["months"] == "0"


def test_WydawnictwoPBNAdapter_autor_isbn_eisbn(
    pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina,
):
    praca_z_dyscyplina_pbn = pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina

    praca_z_dyscyplina_pbn.isbn = None
    praca_z_dyscyplina_pbn.e_isbn = "123"

    res = WydawnictwoPBNAdapter(praca_z_dyscyplina_pbn).pbn_get_json()
    assert res["isbn"] == "123"

    praca_z_dyscyplina_pbn.isbn = "456"
    praca_z_dyscyplina_pbn.e_isbn = None

    res = WydawnictwoPBNAdapter(praca_z_dyscyplina_pbn).pbn_get_json()
    assert res["isbn"] == "456"

    praca_z_dyscyplina_pbn.isbn = "789"
    praca_z_dyscyplina_pbn.e_isbn = "123"

    res = WydawnictwoPBNAdapter(praca_z_dyscyplina_pbn).pbn_get_json()
    assert res["isbn"] == "789"


def test_WydawnictwoPBNAdapter_autor_z_orcid_bez_dyscypliny_idzie_bez_id(
    praca_z_dyscyplina_pbn, jednostka, denorms
):
    pierwszy_autor = praca_z_dyscyplina_pbn.autorzy_set.first().autor
    pierwszy_autor.orcid = "123456"
    pierwszy_autor.save()

    autor_bez_dyscypliny = baker.make(
        Autor, imiona="Jan", nazwisko="Budnik", orcid="43567"
    )
    praca_z_dyscyplina_pbn.dodaj_autora(autor_bez_dyscypliny, jednostka, "Jan Budnik")

    res = WydawnictwoPBNAdapter(praca_z_dyscyplina_pbn).pbn_get_json()
    assert res["authors"][0].get("orcidId")
    assert not res["authors"][1].get("orcidId")


def test_WydawnictwoPBNAdapter_pod_redakcja_falsz(
    ksiazka, autor_jan_nowak, jednostka, denorms
):
    ksiazka.dodaj_autora(autor_jan_nowak, jednostka)
    assert WydawnictwoPBNAdapter(ksiazka).pod_redakcja() is False


def test_WydawnictwoPBNAdapter_pod_redakcja_prawda(
    ksiazka, autor_jan_nowak, jednostka, denorms
):
    ksiazka.dodaj_autora(autor_jan_nowak, jednostka, typ_odpowiedzialnosci_skrot="red.")
    assert WydawnictwoPBNAdapter(ksiazka).pod_redakcja() is True


def test_WydawnictwoPBNAdapter_slowa_kluczowe(pbn_wydawnictwo_zwarte_ksiazka):
    pbn_wydawnictwo_zwarte_ksiazka.slowa_kluczowe.add("test")
    pbn_wydawnictwo_zwarte_ksiazka.slowa_kluczowe.add("best")

    ret = WydawnictwoPBNAdapter(pbn_wydawnictwo_zwarte_ksiazka).pbn_get_json()
    assert "best" in ret["languageData"]["keywords"][0]["keywords"]


def test_WydawnictwoPBNAdapter_streszczenia(pbn_wydawnictwo_zwarte_ksiazka):
    pbn_wydawnictwo_zwarte_ksiazka.streszczenia.create(
        jezyk_streszczenia=None, streszczenie="123 test streszczenia"
    )
    ret = WydawnictwoPBNAdapter(pbn_wydawnictwo_zwarte_ksiazka).pbn_get_json()
    assert "test" in ret["languageData"]["abstracts"][0]["text"]


def test_WydawnictwoPBNAdapter_wydawnictwo_zwarte_strony(
    pbn_wydawnictwo_zwarte_ksiazka,
):
    pbn_wydawnictwo_zwarte_ksiazka.strony = "pincset"

    ret = WydawnictwoPBNAdapter(pbn_wydawnictwo_zwarte_ksiazka).pbn_get_json()
    assert "pagesFromTo" in ret


def test_WydawnictwoPBNAdapter_wydawnictwo_ciagle_strony(
    pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina,
):
    pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina.strony = "pincset"
    ret = WydawnictwoPBNAdapter(
        pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina
    ).pbn_get_json()
    assert "pagesFromTo" in ret


def test_WydawnictwoPBNAdapter_wydawnictwo_ciagle_zakres_stron(
    pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina,
):
    pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina.szczegoly = "s. 20-30"
    ret = WydawnictwoPBNAdapter(
        pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina
    ).pbn_get_json()
    assert "pagesFromTo" in ret


def encode_then_decode_json(in_json):
    return json.loads(json.dumps(in_json))


def test_WydawnictwoPBNAdapter_oplata_za_publikacje_platna(
    pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina,
):
    pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina.opl_pub_amount = Decimal("50.4")
    pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina.opl_pub_research_potential = True

    ret = encode_then_decode_json(
        WydawnictwoPBNAdapter(
            pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina
        ).pbn_get_json()
    )
    assert ret["fee"]["amount"] == "50.4"
    assert ret["fee"]["researchPotentialFinancialResources"]


def test_WydawnictwoPBNAdapter_oplata_za_publikacje_darmowa(
    pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina,
):
    pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina.opl_pub_cost_free = True

    ret = encode_then_decode_json(
        WydawnictwoPBNAdapter(
            pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina
        ).pbn_get_json()
    )
    assert ret["fee"]["amount"] == Decimal("0")
    assert ret["fee"]["costFreePublication"]



import pytest
from django.db import connection
from model_bakery import baker

from long_running.models import Report
from test_bpp.models import TestOperation


@pytest.fixture
def operation(admin_user):
    return TestOperation.objects.create(owner=admin_user)


@pytest.fixture
def report(db):
    class TestReport(Report):
        sent_notifications = []

        def send_notification(self, *args, **kw):
            self.sent_notifications.append((args, kw))

    # Create the schema for our test model
    with connection.schema_editor() as schema_editor:
        schema_editor.create_model(TestReport)

    yield baker.make(TestReport)

    # with connection.schema_editor() as schema_editor:
    #     schema_editor.delete_model(TestReport)



# -*- coding: utf-8 -*-


from django.db import migrations, models

def ustaw_obca_jednostka(apps, schema_editor):
    Jednostka = apps.get_model("bpp", "Jednostka")
    Uczelnia = apps.get_model("bpp", "Uczelnia")

    try:
        j = Jednostka.objects.get(nazwa="Obca jednostka")
    except Jednostka.DoesNotExist:
        return

    u = Uczelnia.objects.all().first()
    u.obca_jednostka = j
    u.save()

def ustaw_obca_jednostka_reverse(*args, **kw):
    pass

class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0050_usun_model_historyczny'),
    ]

    operations = [
        migrations.AddField(
            model_name='uczelnia',
            name='obca_jednostka',
            field=models.ForeignKey(on_delete=models.CASCADE, blank=True, to='bpp.Jednostka', help_text=b'\n    Jednostka skupiaj\xc4\x85ca autor\xc3\xb3w nieindeksowanych, nie b\xc4\x99d\xc4\x85cych pracownikami uczelni. Procedury importuj\xc4\x85ce\n    dane z zewn\xc4\x99trznych system\xc3\xb3w informatycznych b\xc4\x99d\xc4\x85 przypisywa\xc4\x87 do tej jednostki osoby, kt\xc3\xb3re zako\xc5\x84czy\xc5\x82y\n    prac\xc4\x99 na uczelni. ', null=True),
        ),
        migrations.RunPython(ustaw_obca_jednostka, ustaw_obca_jednostka_reverse)
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-11-06 10:35
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Article',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('status', model_utils.fields.StatusField(choices=[('draft', 'draft'), ('published', 'published')], default='draft', max_length=100, no_check_for_status=True, verbose_name='status')),
                ('status_changed', model_utils.fields.MonitorField(default=django.utils.timezone.now, monitor='status', verbose_name='status changed')),
                ('title', models.TextField(verbose_name='Title')),
                ('article_body', model_utils.fields.SplitField(help_text='Use the split marker "&lt;!-- tutaj --&gt;" in case you want to displaythe shorter version of the article body', no_excerpt_field=True, verbose_name='Article body')),
                ('published_on', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Published on')),
                ('slug', models.SlugField(unique=True)),
                ('_article_body_excerpt', models.TextField(editable=False)),
            ],
            options={
                'verbose_name': 'Article',
                'verbose_name_plural': 'Articles',
                'ordering': ('-published_on', 'title'),
            },
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import migrations, models

STARE_DANE = {}

def ustaw_skupia_pracownikow_zarzadzaj_automatycznie(apps, schema_editor):
    Jednostka = apps.get_model("bpp", "Jednostka")
    for j in Jednostka.objects.all():
        if STARE_DANE[j.pk]['wirtualna']:
            j.zarzadzaj_automatycznie = False
        if STARE_DANE[j.pk]['obca_jednostka']:
            j.skupia_pracownikow = False
        j.save()
    pass

def nic(apps, schema_editor):
    pass

def zapisz_tymczasowe_dane_w_zmiennej_globalnej(apps, schema_editor):
    Jednostka = apps.get_model("bpp", "Jednostka")
    for j in Jednostka.objects.all():
        STARE_DANE[j.pk] = {'wirtualna': j.wirtualna, 'obca_jednostka': j.obca_jednostka}

class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0051_uczelnia_obca_jednostka'),
    ]

    operations = [
        migrations.RunPython(
            zapisz_tymczasowe_dane_w_zmiennej_globalnej,
            nic
        ),

        migrations.AddField(
            model_name='jednostka',
            name='skupia_pracownikow',
            field=models.BooleanField(default=True, help_text=b"Ta jednostka skupia osoby b\xc4\x99d\xc4\x85ce faktycznymi pracownikami uczelni. Odznacz dla jednostek\n         typu 'Studenci', 'Doktoranci', 'Pracownicy emerytowani' itp.", verbose_name=b'Skupia pracownik\xc3\xb3w'),
        ),
        migrations.AddField(
            model_name='jednostka',
            name='zarzadzaj_automatycznie',
            field=models.BooleanField(default=True, help_text=b'Jednostka ta b\xc4\x99dzie dowolnie modyfikowana przez procedury importujace dane z zewn\xc4\x99trznych\n        system\xc3\xb3w informatycznych', verbose_name=b'Zarz\xc4\x85dzaj automatycznie'),
        ),
        migrations.RemoveField(
            model_name='jednostka',
            name='nie_archiwizuj',
        ),
        migrations.RemoveField(
            model_name='jednostka',
            name='obca_jednostka',
        ),
        migrations.RemoveField(
            model_name='jednostka',
            name='wirtualna',
        ),
        migrations.RunPython(
            ustaw_skupia_pracownikow_zarzadzaj_automatycznie,
            nic
        ),

    ]



from rest_framework import viewsets

from api_v1.serializers.nagroda import NagrodaSerializer
from bpp.models.nagroda import Nagroda


class NagrodaViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = Nagroda.objects.all()
    serializer_class = NagrodaSerializer



# Generated by Django 3.0.11 on 2021-03-22 23:25

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
    ]

    operations = [
        migrations.CreateModel(
            name="IgnorujRozbieznoscIf",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("object_id", models.PositiveIntegerField(db_index=True)),
                (
                    "content_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="contenttypes.ContentType",
                    ),
                ),
            ],
        ),
    ]



# -*- encoding: utf-8 -*-

from celery.utils.log import get_task_logger
from django.core.management import call_command

try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse

from requests.exceptions import ConnectionError

from bpp.util import remove_old_objects
from integrator2.models.lista_ministerialna import ListaMinisterialnaIntegration
from long_running.util import wait_for_object

logger = get_task_logger(__name__)
from django_bpp.celery_tasks import app


@app.task
def analyze_file(pk):
    obj = wait_for_object(ListaMinisterialnaIntegration, pk)

    def informuj(komunikat, dont_persist=True):
        try:
            msg = '<a href="%s">Integracja pliku "%s": %s</a>. '
            url = reverse(
                "integrator2:detail",
                args=(
                    obj._meta.model_name,
                    obj.pk,
                ),
            )
            call_command(
                "send_message",
                obj.owner,
                msg % (url, obj.filename(), komunikat),
                dont_persist=dont_persist,
            )
        except ConnectionError:
            pass
        except Exception as e:
            obj.extra_info = str(e)
            obj.status = 3
            obj.save()
            raise

    obj.status = 1
    obj.save()

    try:
        obj.dict_stream_to_db()
    except Exception as e:
        obj.extra_info = str(e)
        obj.status = 3
        obj.save()

        informuj("wystąpił błąd na etapie importu danych")
        raise

    informuj("zaimportowano, trwa analiza danych")

    try:
        obj.match_records()
    except Exception as e:
        obj.extra_info = str(e)
        obj.status = 3
        obj.save()

        informuj("wystąpił błąd na etapie matchowania danych")
        raise

    informuj("rozpoczęto integrację")

    try:
        obj.integrate()
    except Exception as e:
        obj.extra_info = str(e)
        obj.status = 3
        obj.save()
        informuj("wystąpił błąd na etapie integracji danych")
        raise

    obj.status = 2
    obj.save()

    informuj("zakończono integrację", dont_persist=False)


@app.task
def remove_old_integrator_files():
    return remove_old_objects(ListaMinisterialnaIntegration, field_name="uploaded_on")



from model_bakery import baker

from long_running import const
from long_running.views import (
    CreateLongRunningOperationView,
    LongRunningOperationsView,
    LongRunningResultsView,
    LongRunningRouterView,
    LongRunningSingleObjectChannelSubscriberMixin,
    LongRunningTaskCallerMixin,
    RestartLongRunningOperationView,
    RestrictToOwnerMixin,
)
from test_bpp.models import TestOperation

from bpp.system import User


def test_LongRunningTaskCallerMixin_task_on_commit(
    mocker,
    admin_user,
):
    class Foo(LongRunningTaskCallerMixin):
        model = User
        task = mocker.MagicMock()

    toc = mocker.patch("long_running.views.transaction")
    Foo.task_on_commit(Foo, admin_user.pk)
    toc.on_commit.assert_called_once()


def test_RestrictToOwnerMixin(mocker):
    class Foo(RestrictToOwnerMixin):
        model = mocker.MagicMock()
        request = mocker.MagicMock()

    Foo.get_queryset(Foo)
    assert Foo.model.objects.filter.called_with(owner=Foo.request)


def test_LongRunningOperationsView_get_queryset(rf, admin_user):
    class TestLongRunningOperationsView(LongRunningOperationsView):
        model = TestOperation

        class request:
            user = admin_user

    v = TestLongRunningOperationsView()

    baker.make(TestOperation, owner=admin_user)
    rf.user = admin_user
    qset = v.get_queryset()
    assert qset.count() == 1


def test_LongRunningSingleObjectChannelSubscriberMixin(rf, admin_user):
    class Bar:
        def get_context_data(self, *args, **kw):
            return kw

    class Foo(LongRunningSingleObjectChannelSubscriberMixin, Bar):
        class object:
            pk = 5

    v = Foo().get_context_data()
    assert v["extraChannels"] == [5]


def test_LongRunningRouterView_get_not_started():
    class Foo(LongRunningRouterView):
        def get_object(self):
            class ret:
                def get_state(self):
                    return const.PROCESSING_NOT_STARTED

            return ret()

        request = 123

        def get_context_data(self):
            return {}

    assert Foo().get() is not None


def test_LongRunningRouterView_get_started():
    class Foo(LongRunningRouterView):
        def get_object(self):
            class ret:
                def get_state(self):
                    return const.PROCESSING_STARTED

                def get_url(self, arg):
                    return "foo"

                pk = 100

            return ret()

    assert Foo().get().status_code == 302


def test_LongRunningResultsView_parent_obejct(operation):
    class Foo(LongRunningResultsView):
        model = TestOperation
        kwargs = dict(pk=operation.pk)

        class request:
            user = operation.owner

    assert Foo().parent_object == operation


def test_LongRunningResultsView_parent_object_get_details_set(
    wydawnictwo_ciagle, mocker
):
    class Foo(LongRunningResultsView):
        parent_object = wydawnictwo_ciagle

    wydawnictwo_ciagle.get_details_set = mocker.MagicMock()

    Foo().get_queryset()

    wydawnictwo_ciagle.get_details_set.assert_called_once()


def test_CreateLongRunningOperationView(wydawnictwo_ciagle, mocker):
    class Foo(CreateLongRunningOperationView):
        parent_object = wydawnictwo_ciagle

        def get_success_url(self):
            return "120"

        def task_on_commit(self, pk):
            return

        class request:
            user = 123

    class form:
        class instance:
            pk = 5
            owner = None

        def save():
            return wydawnictwo_ciagle

    assert Foo().form_valid(form).status_code == 302
    assert form.instance.owner == 123


def test_RestartLongRunningOperationView_get(operation, mocker):
    obj = mocker.MagicMock()

    class Foo(RestartLongRunningOperationView):
        def get_object(self):
            return obj

        task_on_commit = mocker.MagicMock()

    foo = Foo()
    assert foo.get().status_code == 302
    obj.mark_reset.assert_called_once()
    foo.task_on_commit.assert_called_once()



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-19 09:42
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0089_legacy_data'),
    ]

    operations = [
        migrations.AddField(
            model_name='patent',
            name='pbn_id',
            field=models.IntegerField(blank=True, db_index=True, help_text='Identyfikator w systemie Polskiej Bibliografii Naukowej (PBN)', null=True, unique=True, verbose_name='Identyfikator PBN'),
        ),
        migrations.AddField(
            model_name='praca_doktorska',
            name='pbn_id',
            field=models.IntegerField(blank=True, db_index=True, help_text='Identyfikator w systemie Polskiej Bibliografii Naukowej (PBN)', null=True, unique=True, verbose_name='Identyfikator PBN'),
        ),
        migrations.AddField(
            model_name='praca_habilitacyjna',
            name='pbn_id',
            field=models.IntegerField(blank=True, db_index=True, help_text='Identyfikator w systemie Polskiej Bibliografii Naukowej (PBN)', null=True, unique=True, verbose_name='Identyfikator PBN'),
        ),
        migrations.AddField(
            model_name='wydawnictwo_ciagle',
            name='pbn_id',
            field=models.IntegerField(blank=True, db_index=True, help_text='Identyfikator w systemie Polskiej Bibliografii Naukowej (PBN)', null=True, unique=True, verbose_name='Identyfikator PBN'),
        ),
        migrations.AddField(
            model_name='wydawnictwo_zwarte',
            name='pbn_id',
            field=models.IntegerField(blank=True, db_index=True, help_text='Identyfikator w systemie Polskiej Bibliografii Naukowej (PBN)', null=True, unique=True, verbose_name='Identyfikator PBN'),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import migrations, models
from django.contrib.postgres.search import SearchVectorField


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0036_auto_20160117_2147'),
    ]

    operations = [
        migrations.AlterField(
            model_name='autor',
            name='search',
            field=SearchVectorField(),
        ),
        migrations.AlterField(
            model_name='jednostka',
            name='search',
            field=SearchVectorField(null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='patent',
            name='search_index',
            field=SearchVectorField(),
        ),
        migrations.AlterField(
            model_name='praca_doktorska',
            name='search_index',
            field=SearchVectorField(),
        ),
        migrations.AlterField(
            model_name='praca_habilitacyjna',
            name='search_index',
            field=SearchVectorField(),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle',
            name='search_index',
            field=SearchVectorField(),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='search_index',
            field=SearchVectorField(),
        ),
        migrations.AlterField(
            model_name='zrodlo',
            name='search',
            field=SearchVectorField(),
        ),
    ]



from django.db.models import Q

from import_common.normalization import (
    normalize_doi,
    normalize_isbn,
    normalize_public_uri,
    normalize_tytul_publikacji,
)
from pbn_api.models import Publication


def matchuj_pbn_api_publication(title, year, doi, public_uri, isbn, zrodlo):

    title = normalize_tytul_publikacji(title)
    query = Q(title__istartswith=title, year=year)

    doi = normalize_doi(doi)
    if doi:
        query |= Q(doi=doi, year=year)

    public_uri = normalize_public_uri(public_uri)
    if public_uri:
        query |= Q(publicUri=public_uri, year=year)

    isbn = normalize_isbn(isbn)
    if isbn:
        query |= Q(isbn=isbn, year=year)

    res = Publication.objects.filter(query)
    if res.count() == 1:
        return res.first()
    else:
        if zrodlo is not None and zrodlo.pbn_uid_id is not None:
            print(f"Szukam po zrodle, {res.count()}")
            for elem in res:
                if elem.journal and elem.journal["id"] == zrodlo.pbn_uid_id:
                    return elem



# Generated by Django 2.1.10 on 2019-10-30 06:06

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0190_auto_20191030_0532'),
    ]

    operations = [
        migrations.CreateModel(
            name='Wydawnictwo_Zwarte_Zewnetrzna_Baza_Danych',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('info', models.CharField(blank=True, max_length=512, null=True, verbose_name='Informacje dodatkowe')),
                ('baza', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bpp.Zewnetrzna_Baza_Danych')),
                ('rekord', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='zewnetrzna_baza_danych', to='bpp.Wydawnictwo_Zwarte')),
            ],
            options={
                'verbose_name': 'powiązanie wydawnictwa zwartego z zewnętrznymi bazami danych',
                'verbose_name_plural': 'powiązania wydawnictw zwartych z zewnętrznymi bazami danych',
            },
        ),
    ]



# Generated by Django 3.0.14 on 2021-10-28 00:57

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0306_delete_ewaluacja2021liczbandlauczelni"),
    ]

    operations = [
        migrations.CreateModel(
            name="Wydawnictwo_Zwarte_Streszczenie",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("streszczenie", models.TextField(blank=True, null=True)),
                (
                    "jezyk_streszczenia",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="bpp.Jezyk",
                    ),
                ),
                (
                    "rekord",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="bpp.Wydawnictwo_Zwarte",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Wydawnictwo_Ciagle_Streszczenie",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("streszczenie", models.TextField(blank=True, null=True)),
                (
                    "jezyk_streszczenia",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="bpp.Jezyk",
                    ),
                ),
                (
                    "rekord",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="bpp.Wydawnictwo_Ciagle",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



from dal import autocomplete
from django import forms
from django.forms.utils import flatatt
from djangoql.admin import DjangoQLSearchMixin
from mptt.forms import TreeNodeChoiceField
from taggit.forms import TextareaTagWidget

from pbn_api.models import Publication
from . import BaseBppAdminMixin
from .actions import (
    ustaw_po_korekcie,
    ustaw_przed_korekta,
    ustaw_w_trakcie_korekty,
    wyslij_do_pbn,
)
from .core import KolumnyZeSkrotamiMixin, generuj_inline_dla_autorow

# Widget do automatycznego uzupełniania punktacji wydawnictwa ciągłego
from .element_repozytorium import Element_RepozytoriumInline
from .grant import Grant_RekorduInline
from .helpers import (
    MODEL_OPCJONALNIE_NIE_EKSPORTOWANY_DO_API_FIELDSET,
    MODEL_Z_OPLATA_ZA_PUBLIKACJE_FIELDSET,
    OptionalPBNSaveMixin,
    sprawdz_duplikaty_www_doi,
)
from .xlsx_export import resources
from .xlsx_export.mixins import EksportDanychMixin

from django.contrib import admin

from django.utils.safestring import mark_safe

from bpp.admin.filters import (
    DOIUstawioneFilter,
    LiczbaZnakowFilter,
    OstatnioZmienionePrzezFilter,
    PBN_UID_IDObecnyFilter,
    UtworzonePrzezFilter,
)
from bpp.admin.helpers import (
    ADNOTACJE_Z_DATAMI_ORAZ_PBN_FIELDSET,
    DWA_TYTULY,
    EKSTRA_INFORMACJE_WYDAWNICTWO_CIAGLE_FIELDSET,
    MODEL_PUNKTOWANY_KOMISJA_CENTRALNA_FIELDSET,
    MODEL_PUNKTOWANY_WYDAWNICTWO_CIAGLE_FIELDSET,
    MODEL_TYPOWANY_FIELDSET,
    MODEL_Z_ROKIEM,
    MODEL_ZE_SZCZEGOLAMI,
    NIZSZE_TEXTFIELD_Z_MAPA_ZNAKOW,
    OPENACCESS_FIELDSET,
    POZOSTALE_MODELE_WYDAWNICTWO_CIAGLE_FIELDSET,
    PRACA_WYBITNA_FIELDSET,
    PRZED_PO_LISCIE_AUTOROW_FIELDSET,
    AdnotacjeZDatamiOrazPBNMixin,
    DomyslnyStatusKorektyMixin,
    sprobuj_policzyc_sloty,
)
from bpp.admin.nagroda import NagrodaInline
from bpp.models import (  # Publikacja_Habilitacyjna
    Charakter_Formalny,
    Wydawnictwo_Ciagle,
    Wydawnictwo_Ciagle_Streszczenie,
    Wydawnictwo_Ciagle_Zewnetrzna_Baza_Danych,
    Zrodlo,
    nie_zawiera_adresu_doi_org,
    nie_zawiera_http_https,
)

# Proste tabele
from bpp.models.konferencja import Konferencja
from bpp.models.wydawnictwo_ciagle import Wydawnictwo_Ciagle_Autor

#
# Wydaniwcto Ciągłe
#


class Button(forms.Widget):
    """
    A widget that handles a submit button.
    """

    def render(self, name, value, attrs=None, renderer=None):
        final_attrs = self.build_attrs(self.attrs, dict(type="button", name=name))

        return mark_safe(
            '<input type="button"%s value="%s" />'
            % (
                flatatt(final_attrs),
                final_attrs["label"],
            )
        )


class CleanDOIWWWPublicWWWMixin:
    def clean_www(self):
        v = self.cleaned_data.get("www")
        nie_zawiera_adresu_doi_org(v)
        return v

    def clean_public_www(self):
        v = self.cleaned_data.get("public_www")
        nie_zawiera_adresu_doi_org(v)
        return v

    def clean_doi(self):
        v = self.cleaned_data.get("doi")
        nie_zawiera_http_https(v)
        return v


class Wydawnictwo_CiagleForm(CleanDOIWWWPublicWWWMixin, forms.ModelForm):
    zrodlo = forms.ModelChoiceField(
        queryset=Zrodlo.objects.all(),
        widget=autocomplete.ModelSelect2(url="bpp:admin-zrodlo-autocomplete"),
    )

    uzupelnij_punktacje = forms.CharField(
        initial=None,
        max_length=50,
        required=False,
        label="Uzupełnij punktację",
        widget=Button(
            dict(
                id="id_uzupelnij_punktacje",
                label="Uzupełnij punktację",
            )
        ),
    )

    konferencja = forms.ModelChoiceField(
        required=False,
        queryset=Konferencja.objects.all(),
        widget=autocomplete.ModelSelect2(
            url="bpp:konferencja-autocomplete", attrs=dict(style="width: 746px;")
        ),
    )

    pbn_uid = forms.ModelChoiceField(
        label="Odpowiednik w PBN",
        required=False,
        queryset=Publication.objects.all(),
        widget=autocomplete.ModelSelect2(
            url="bpp:publication-autocomplete", attrs=dict(style="width: 746px;")
        ),
    )

    charakter_formalny = TreeNodeChoiceField(
        required=True, queryset=Charakter_Formalny.objects.all()
    )

    status_korekty = DomyslnyStatusKorektyMixin.status_korekty

    class Meta:
        fields = "__all__"

        widgets = {
            "strony": forms.TextInput(attrs=dict(style="width: 150px")),
            "tom": forms.TextInput(attrs=dict(style="width: 150px")),
            "nr_zeszytu": forms.TextInput(attrs=dict(style="width: 150px")),
            "slowa_kluczowe": TextareaTagWidget(attrs={"rows": 2}),
        }


class Wydawnictwo_Ciagle_Zewnetrzna_Baza_DanychForm(forms.ModelForm):
    class Meta:
        fields = ["baza", "info"]


class Wydawnictwo_Ciagle_StreszczenieInline(admin.StackedInline):
    model = Wydawnictwo_Ciagle_Streszczenie
    extra = 0
    fields = ["jezyk_streszczenia", "streszczenie"]


class Wydawnictwo_Ciagle_Zewnetrzna_Baza_DanychInline(admin.StackedInline):
    model = Wydawnictwo_Ciagle_Zewnetrzna_Baza_Danych
    extra = 0
    form = Wydawnictwo_Ciagle_Zewnetrzna_Baza_DanychForm


class Wydawnictwo_CiagleAdmin(
    DjangoQLSearchMixin,
    OptionalPBNSaveMixin,
    KolumnyZeSkrotamiMixin,
    AdnotacjeZDatamiOrazPBNMixin,
    BaseBppAdminMixin,
    EksportDanychMixin,
    admin.ModelAdmin,
):
    resource_class = resources.Wydawnictwo_CiagleResource

    djangoql_completion_enabled_by_default = False
    djangoql_completion = True

    formfield_overrides = NIZSZE_TEXTFIELD_Z_MAPA_ZNAKOW
    actions = [
        ustaw_po_korekcie,
        ustaw_w_trakcie_korekty,
        ustaw_przed_korekta,
        wyslij_do_pbn,
    ]

    form = Wydawnictwo_CiagleForm

    ordering = ("-ostatnio_zmieniony",)

    list_display = [
        "tytul_oryginalny",
        "zrodlo_col",
        "rok",
        "typ_kbn__skrot",
        "charakter_formalny__skrot",
        "liczba_znakow_wydawniczych",
        "ostatnio_zmieniony",
    ]

    list_select_related = ["zrodlo", "typ_kbn", "charakter_formalny"]

    search_fields = [
        "tytul",
        "tytul_oryginalny",
        "szczegoly",
        "uwagi",
        "informacje",
        "slowa_kluczowe__name",
        "rok",
        "id",
        "issn",
        "e_issn",
        "zrodlo__nazwa",
        "zrodlo__skrot",
        "adnotacje",
        "liczba_znakow_wydawniczych",
        "konferencja__nazwa",
        "doi",
        "pbn_uid__pk",
    ]

    list_filter = [
        "status_korekty",
        "recenzowana",
        "typ_kbn",
        "charakter_formalny",
        "jezyk",
        LiczbaZnakowFilter,
        "rok",
        DOIUstawioneFilter,
        "openaccess_tryb_dostepu",
        "openaccess_wersja_tekstu",
        "openaccess_licencja",
        "openaccess_czas_publikacji",
        OstatnioZmienionePrzezFilter,
        UtworzonePrzezFilter,
        PBN_UID_IDObecnyFilter,
    ]

    autocomplete_fields = [
        "pbn_uid",
    ]

    fieldsets = (
        (
            "Wydawnictwo ciągłe",
            {
                "fields": DWA_TYTULY
                + (
                    "zrodlo",
                    "konferencja",
                )
                + MODEL_ZE_SZCZEGOLAMI
                + ("nr_zeszytu",)
                + MODEL_Z_ROKIEM
            },
        ),
        EKSTRA_INFORMACJE_WYDAWNICTWO_CIAGLE_FIELDSET,
        MODEL_TYPOWANY_FIELDSET,
        MODEL_PUNKTOWANY_WYDAWNICTWO_CIAGLE_FIELDSET,
        MODEL_PUNKTOWANY_KOMISJA_CENTRALNA_FIELDSET,
        POZOSTALE_MODELE_WYDAWNICTWO_CIAGLE_FIELDSET,
        ADNOTACJE_Z_DATAMI_ORAZ_PBN_FIELDSET,
        MODEL_OPCJONALNIE_NIE_EKSPORTOWANY_DO_API_FIELDSET,
        OPENACCESS_FIELDSET,
        PRACA_WYBITNA_FIELDSET,
        PRZED_PO_LISCIE_AUTOROW_FIELDSET,
        MODEL_Z_OPLATA_ZA_PUBLIKACJE_FIELDSET,
    )

    inlines = (
        generuj_inline_dla_autorow(Wydawnictwo_Ciagle_Autor),
        NagrodaInline,
        Wydawnictwo_Ciagle_Zewnetrzna_Baza_DanychInline,
        Grant_RekorduInline,
        Element_RepozytoriumInline,
        Wydawnictwo_Ciagle_StreszczenieInline,
    )

    def zrodlo_col(self, obj):
        if obj.zrodlo:
            try:
                return obj.zrodlo.nazwa
            except Zrodlo.DoesNotExist:
                return ""

    zrodlo_col.admin_order_field = "zrodlo__nazwa"
    zrodlo_col.short_description = "Źródło"

    def save_model(self, request, obj, form, change):
        super().save_model(request, obj, form, change)
        sprobuj_policzyc_sloty(request, obj)
        sprawdz_duplikaty_www_doi(request, obj)


admin.site.register(Wydawnictwo_Ciagle, Wydawnictwo_CiagleAdmin)



# Generated by Django 3.0.14 on 2021-05-19 00:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0265_rekord_mat_doi"),
    ]

    operations = [
        migrations.AddField(
            model_name="autor",
            name="pokazuj_poprzednie_nazwiska",
            field=models.BooleanField(
                default=True,
                help_text="Jeżeli odznaczone, poprzednie nazwiska nie będą się wyświetlać na podstronie autora dla użytkowników niezalogowanych. Użytkownicy zalogowani widzą je zawsze. Wyszukiwanie po poprzednich nazwiskach będzie nadal możliwe. ",
            ),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0005_auto_20141107_1950'),
    ]

    operations = [
        migrations.AddField(
            model_name='praca_doktorska',
            name='promotor',
            field=models.ForeignKey(on_delete=models.CASCADE, related_name='promotor_doktoratu', blank=True, to='bpp.Autor', null=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='patent',
            name='afiliowana',
            field=models.BooleanField(default=False, db_index=True),
        ),
        migrations.AlterField(
            model_name='patent',
            name='recenzowana',
            field=models.BooleanField(default=False, db_index=True),
        ),
        migrations.AlterField(
            model_name='praca_doktorska',
            name='afiliowana',
            field=models.BooleanField(default=False, db_index=True),
        ),
        migrations.AlterField(
            model_name='praca_doktorska',
            name='recenzowana',
            field=models.BooleanField(default=False, db_index=True),
        ),
        migrations.AlterField(
            model_name='praca_habilitacyjna',
            name='afiliowana',
            field=models.BooleanField(default=False, db_index=True),
        ),
        migrations.AlterField(
            model_name='praca_habilitacyjna',
            name='recenzowana',
            field=models.BooleanField(default=False, db_index=True),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle',
            name='afiliowana',
            field=models.BooleanField(default=False, db_index=True),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle',
            name='recenzowana',
            field=models.BooleanField(default=False, db_index=True),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='afiliowana',
            field=models.BooleanField(default=False, db_index=True),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='recenzowana',
            field=models.BooleanField(default=False, db_index=True),
        ),
    ]



# Generated by Django 3.0.14 on 2022-01-09 00:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("tee", "0003_log_traceback"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="log",
            name="exit_code",
        ),
        migrations.RemoveField(
            model_name="log",
            name="exit_value",
        ),
        migrations.RemoveField(
            model_name="log",
            name="kwargs",
        ),
        migrations.AddField(
            model_name="log",
            name="finished_successfully",
            field=models.NullBooleanField(),
        ),
    ]



import pytest

from nowe_raporty.views import (
    GenerujRaportDlaAutora,
    GenerujRaportDlaJednostki,
    GenerujRaportDlaUczelni,
    GenerujRaportDlaWydzialu,
)


@pytest.mark.parametrize("par", [True, False])
def test_view_GenerujRaportDlaUczelni_get_base_queryset(uczelnia, rf, par):
    v = GenerujRaportDlaUczelni()

    v.request = rf.get("/", args={"_tzju": str(par)})
    assert list(v.get_base_queryset()) == []


@pytest.mark.parametrize("par", [True, False])
def test_view_GenerujRaportDlaWydzialu_get_base_queryset(wydzial, rf, par):
    v = GenerujRaportDlaWydzialu()
    v.object = wydzial

    v.request = rf.get("/", args={"_tzju": str(par)})
    assert list(v.get_base_queryset()) == []


@pytest.mark.parametrize("par", [True, False])
def test_view_GenerujRaportDlaJednostki_get_base_queryset(jednostka, rf, par):
    v = GenerujRaportDlaJednostki()
    v.object = jednostka

    v.request = rf.get("/", args={"_tzju": str(par)})
    assert list(v.get_base_queryset()) == []


@pytest.mark.parametrize("par", [True, False])
def test_view_GenerujRaportDlaAutora_get_base_queryset(autor, rf, par):
    v = GenerujRaportDlaAutora()
    v.object = autor

    v.request = rf.get("/", args={"_tzju": str(par)})
    assert list(v.get_base_queryset()) == []



# Generated by Django 3.0.14 on 2021-08-16 06:17

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("pbn_api", "0025_auto_20210809_0149"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="oswiadczenieinstytucji",
            unique_together=set(),
        ),
        migrations.AlterUniqueTogether(
            name="publikacjainstytucji",
            unique_together=set(),
        ),
    ]



# Generated by Django 2.2.7 on 2020-01-06 15:53

from django.db import migrations, models
import django.db.models.deletion

from bpp.models import Uczelnia

class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0192_auto_20191107_0853'),
    ]

    operations = [
        migrations.RunSQL("DELETE FROM bpp_cache_punktacja_autora;"),
        migrations.AddField(
            model_name='cache_punktacja_autora',
            name='jednostka',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bpp.Jednostka'),
            preserve_default=False,
        ),
    ]



# Generated by Django 2.1.7 on 2019-08-24 07:48

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0174_rodzaj_pbn_dla_charakteru'),
        ('bpp', '0174_uczelnia_podpowiadaj_dyscypliny_konfig'),
    ]

    operations = [
    ]



# Generated by Django 3.0.14 on 2021-06-06 16:46

import django.db.models.deletion
import mptt.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0274_jednostka_hier"),
    ]

    operations = [
        migrations.AlterField(
            model_name="jednostka",
            name="parent",
            field=mptt.fields.TreeForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="children",
                to="bpp.Jednostka",
                verbose_name="Jednostka nadrzędna",
            ),
        ),
    ]



# -*- encoding: utf-8 -*-

"""
Struktura uczelni.
"""

from autoslug import AutoSlugField
from django.db import models
from django.db.models import CASCADE
from django.urls.base import reverse

from bpp.models import ModelZAdnotacjami
from bpp.models.abstract import ModelZPBN_ID
from bpp.util import FulltextSearchMixin
from .uczelnia import Uczelnia


class Wydzial(ModelZAdnotacjami, ModelZPBN_ID):
    uczelnia = models.ForeignKey(Uczelnia, CASCADE)
    nazwa = models.CharField(
        max_length=512,
        unique=True,
        help_text='Pełna nazwa wydziału, np. "Wydział Lekarski"',
    )
    skrot_nazwy = models.CharField(
        max_length=250,
        unique=True,
        blank=True,
        null=True,
        help_text='Skrót nazwy wydziału, wersja czytelna, np. "Wydz. Lek."',
    )
    skrot = models.CharField(
        "Skrót",
        max_length=10,
        unique=True,
        help_text='Skrót nazwy wydziału, wersja minimalna, np. "WL"',
    )

    opis = models.TextField(null=True, blank=True)
    slug = AutoSlugField(populate_from="nazwa", max_length=512, unique=True)
    poprzednie_nazwy = models.CharField(
        max_length=4096, blank=True, null=True, default=""
    )
    kolejnosc = models.IntegerField("Kolejność", default=0)
    widoczny = models.BooleanField(
        default=True,
        help_text="""Czy wydział ma być widoczny przy przeglądaniu strony dla zakładki "Uczelnia"?""",
    )

    zezwalaj_na_ranking_autorow = models.BooleanField(
        "Zezwalaj na generowanie rankingu autorów dla tego wydziału", default=True
    )

    zarzadzaj_automatycznie = models.BooleanField(
        "Zarządzaj automatycznie",
        default=True,
        help_text="""Wydział ten będzie dowolnie modyfikowany przez procedury importujace dane z zewnętrznych
        systemów informatycznych. W przypadku, gdy pole ma ustawioną wartość na 'fałsz', wydział ten może być""",
    )

    otwarcie = models.DateField("Data otwarcia wydziału", blank=True, null=True)
    zamkniecie = models.DateField("Data zamknięcia wydziału", blank=True, null=True)

    class Meta:
        verbose_name = "wydział"
        verbose_name_plural = "wydziały"
        ordering = ["kolejnosc", "skrot"]
        app_label = "bpp"

    def __str__(self):
        return self.nazwa

    def get_absolute_url(self):
        return reverse("bpp:browse_wydzial", args=(self.slug,))

    def jednostki(self):
        """Lista jednostek - dla WWW"""
        from .jednostka import Jednostka

        return Jednostka.objects.filter(wydzial=self, widoczna=True).order_by(
            *Jednostka.objects.get_default_ordering()
        )


class JednostkaManager(FulltextSearchMixin, models.Manager):
    def create(self, *args, **kw):
        if "wydzial" in kw and not ("uczelnia" in kw or "uczelnia_id" in kw):
            # Kompatybilność wsteczna, z czasów, gdy nie było metryczki historycznej
            # dla obecności jednostki w wydziałach
            kw["uczelnia"] = kw["wydzial"].uczelnia
        return super(JednostkaManager, self).create(*args, **kw)



# Generated by Django 3.0.14 on 2021-09-24 17:39

from django.db import migrations

from bpp.migration_util import load_custom_sql


class Migration(migrations.Migration):

    dependencies = [
        ("rozbieznosci_dyscyplin", "0010_recreate"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="rozbieznosciview",
            options={
                "managed": False,
                "verbose_name": "rozbieżność rekordu i dyscyplin",
                "verbose_name_plural": "rozbieżności rekordów i dyscyplin",
            },
        ),
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql(
                "0011_null_is_wrong", app_name="rozbieznosci_dyscyplin"
            )
        ),
    ]



# Generated by Django 3.2.15 on 2022-08-16 15:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("zglos_publikacje", "0008_auto_20220816_1255"),
    ]

    operations = [
        migrations.AlterField(
            model_name="zgloszenie_publikacji",
            name="status",
            field=models.PositiveSmallIntegerField(
                choices=[
                    (0, "nowe zgłoszenie"),
                    (1, "zaakceptowany - dodany do bazy BPP"),
                    (2, "wymaga zmian - odesłano do zgłaszającego"),
                    (3, "zmiany naniesione przez zgłaszającego"),
                    (4, "odrzucono w całości"),
                    (5, "spam"),
                ],
                default=0,
            ),
        ),
    ]



from __future__ import absolute_import, unicode_literals

import logging
import traceback
from functools import wraps
from importlib import import_module

from django import http
from django.conf import settings
from django.core.exceptions import PermissionDenied
from django.core.handlers.base import BaseHandler
from django.core.serializers.json import DjangoJSONEncoder
from django.core.signals import got_request_exception
from django.utils.module_loading import import_string


class BadRequest(Exception):
    pass


json = import_module(getattr(settings, "JSON_MODULE", "json"))
JSON = getattr(settings, "JSON_DEFAULT_CONTENT_TYPE", "application/json")
logger = logging.getLogger("django.request")


def _dump_json(data):
    options = {}
    options.update(getattr(settings, "JSON_OPTIONS", {}))

    # Use the DjangoJSONEncoder by default, unless cls is set to None.
    options.setdefault("cls", DjangoJSONEncoder)
    if isinstance(options["cls"], str):
        options["cls"] = import_string(options["cls"])
    elif options["cls"] is None:
        options.pop("cls")

    return json.dumps(data, **options)


def json_view(*args, **kwargs):
    """Ensure the response content is well-formed JSON.

    Views wrapped in @json_view can return JSON-serializable Python objects,
    like lists and dicts, and the decorator will serialize the output and set
    the correct Content-type.

    Views may also throw known exceptions.py, like Http404, PermissionDenied, etc,
    and @json_view will convert the response to a standard JSON error format,
    and set the status code and content type.

    If you return a two item tuple, the first is a JSON-serializable object and
    the second is an integer used for the HTTP status code, e.g.:

    >>> @json_view
    ... def example(request):
    ...    return {'foo': 'bar'}, 418

    By default all responses will get application/json as their content type.
    You can override it for non-error responses by giving the content_type
    keyword parameter to the decorator, e.g.:

    >>> @json_view(content_type='application/vnd.example-v1.0+json')
    ... def example2(request):
    ...     return {'foo': 'bar'}

    """

    content_type = kwargs.get("content_type", JSON)

    def decorator(f):
        @wraps(f)
        def _wrapped(request, *a, **kw):
            try:
                status = 200
                headers = {}
                ret = f(request, *a, **kw)

                if isinstance(ret, tuple):
                    if len(ret) == 3:
                        ret, status, headers = ret
                    else:
                        ret, status = ret

                # Some errors are not exceptions.py. :\
                if isinstance(ret, http.HttpResponseNotAllowed):
                    blob = _dump_json(
                        {"error": 405, "message": "HTTP method not allowed."}
                    )
                    return http.HttpResponse(blob, status=405, content_type=JSON)

                # Allow HttpResponses to go straight through.
                if isinstance(ret, http.HttpResponse):
                    return ret

                blob = _dump_json(ret)
                response = http.HttpResponse(
                    blob, status=status, content_type=content_type
                )
                for k in headers:
                    response[k] = headers[k]
                return response
            except http.Http404 as e:
                blob = _dump_json(
                    {
                        "error": 404,
                        "message": str(e),
                    }
                )
                logger.warning(
                    "Not found: %s",
                    request.path,
                    extra={
                        "status_code": 404,
                        "request": request,
                    },
                )
                return http.HttpResponseNotFound(blob, content_type=JSON)
            except PermissionDenied as e:
                logger.warning(
                    "Forbidden (Permission denied): %s",
                    request.path,
                    extra={
                        "status_code": 403,
                        "request": request,
                    },
                )
                blob = _dump_json(
                    {
                        "error": 403,
                        "message": str(e),
                    }
                )
                return http.HttpResponseForbidden(blob, content_type=JSON)
            except BadRequest as e:
                blob = _dump_json(
                    {
                        "error": 400,
                        "message": str(e),
                    }
                )
                return http.HttpResponseBadRequest(blob, content_type=JSON)
            except Exception as e:
                exc_data = {
                    "error": 500,
                    "message": "An error occurred",
                }
                if settings.DEBUG:
                    exc_data["message"] = str(e)
                    exc_data["traceback"] = traceback.format_exc()

                blob = _dump_json(exc_data)

                # Generate the usual 500 error email with stack trace and full
                # debugging information
                logger.error(
                    "Internal Server Error: %s",
                    request.path,
                    exc_info=True,
                    extra={"status_code": 500, "request": request},
                )

                # Here we lie a little bit. Because we swallow the exception,
                # the BaseHandler doesn't get to send this signal. It sets the
                # sender argument to self.__class__, in case the BaseHandler
                # is subclassed.
                got_request_exception.send(
                    sender=BaseHandler, request=request, exception=e, exc_data=exc_data
                )
                return http.HttpResponseServerError(blob, content_type=JSON)

        return _wrapped

    if len(args) == 1 and callable(args[0]):
        return decorator(args[0])
    else:
        return decorator



# Generated by Django 3.0.11 on 2021-01-03 11:52

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0232_auto_20210101_1751"),
    ]

    operations = [
        migrations.AlterField(
            model_name="praca_doktorska",
            name="autor",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE, to="bpp.Autor"
            ),
        ),
    ]



# -*- encoding: utf-8 -*

import messages_extends as messages
from django.contrib.auth import get_user_model

from django.core.management import BaseCommand
from django.test import RequestFactory
from messages_extends.storages import PersistentStorage

import notifications
from notifications.core import _send, send_notification


class Command(BaseCommand):
    help = "Wysyla notyfikacje realtime via django-channels"
    args = "<username> <message>"

    def add_arguments(self, parser):
        parser.add_argument("user")
        parser.add_argument("text")

    def handle(self, *args, **options):
        send_notification(
            options["user"], messages.INFO_PERSISTENT, options["text"],
        )



import json
import os
import random
import time
from datetime import datetime

import django_webtest
import pytest
import webtest
from dbtemplates.models import Template
from django_webtest import DjangoTestApp
from rest_framework.test import APIClient
from splinter.driver import DriverAPI

from pbn_api.models import Language

from bpp.models.szablondlaopisubibliograficznego import SzablonDlaOpisuBibliograficznego
from bpp.util import get_fixture

try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse

from model_bakery import baker

from bpp import const
from bpp.const import GR_RAPORTY_WYSWIETLANIE, GR_WPROWADZANIE_DANYCH, TO_AUTOR
from bpp.fixtures import get_openaccess_data
from bpp.models import (
    Autor_Dyscyplina,
    Dyscyplina_Naukowa,
    Wydawca,
    Zewnetrzna_Baza_Danych,
)
from bpp.models.autor import Autor, Funkcja_Autora, Tytul
from bpp.models.patent import Patent
from bpp.models.praca_doktorska import Praca_Doktorska
from bpp.models.praca_habilitacyjna import Praca_Habilitacyjna
from bpp.models.struktura import Jednostka, Uczelnia, Wydzial
from bpp.models.system import (
    Charakter_Formalny,
    Jezyk,
    Status_Korekty,
    Typ_KBN,
    Typ_Odpowiedzialnosci,
)
from bpp.models.wydawnictwo_ciagle import Wydawnictwo_Ciagle
from bpp.models.wydawnictwo_zwarte import Wydawnictwo_Zwarte
from bpp.models.zrodlo import Zrodlo

from django_bpp.selenium_util import wait_for_page_load, wait_for_websocket_connection

NORMAL_DJANGO_USER_LOGIN = "test_login_bpp"
NORMAL_DJANGO_USER_PASSWORD = "test_password"

from asgi_live_server import asgi_live_server  # noqa

from bpp.tests.util import setup_model_bakery

setup_model_bakery()


def current_rok():
    return datetime.now().date().year


@pytest.fixture
def rok():
    return current_rok()


@pytest.fixture
def dyscyplina1(db):
    return Dyscyplina_Naukowa.objects.get_or_create(
        nazwa="memetyka stosowana", kod="1.1"
    )[0]


@pytest.fixture
def dyscyplina2(db):
    return Dyscyplina_Naukowa.objects.get_or_create(
        nazwa="druga dyscyplina", kod="2.2"
    )[0]


@pytest.fixture
def dyscyplina3(db):
    return Dyscyplina_Naukowa.objects.get_or_create(
        nazwa="trzecia dyscyplina", kod="3.3"
    )[0]


@pytest.fixture
def grupa_raporty_wyswietlanie():
    from django.contrib.auth.models import Group

    return Group.objects.get_or_create(name=GR_RAPORTY_WYSWIETLANIE)[0]


@pytest.fixture
def normal_django_user(request, db, django_user_model):  # , django_username_field):
    """
    A normal Django user
    """

    try:
        obj = django_user_model.objects.get(username=NORMAL_DJANGO_USER_LOGIN)
    except django_user_model.DoesNotExist:
        obj = django_user_model.objects.create_user(
            username=NORMAL_DJANGO_USER_LOGIN, password=NORMAL_DJANGO_USER_PASSWORD
        )

    def fin():
        obj.delete()

    return obj


def _preauth_session_id_helper(
    username,
    password,
    client,
    browser,
    asgi_live_server,  # noqa
    django_user_model,
    django_username_field,
):
    res = client.login(username=username, password=password)
    assert res is True

    with wait_for_page_load(browser):
        browser.visit(asgi_live_server.url + "/non-existant-url")
    browser.cookies.add({"sessionid": client.cookies["sessionid"].value})
    browser.authorized_user = django_user_model.objects.get(
        **{django_username_field: username}
    )
    # with wait_for_page_load(browser):
    #    browser.reload()
    return browser


@pytest.fixture
def preauth_browser(
    normal_django_user,
    client,
    browser,
    asgi_live_server,  # noqa
    django_user_model,
    django_username_field,
):
    browser = _preauth_session_id_helper(
        NORMAL_DJANGO_USER_LOGIN,
        NORMAL_DJANGO_USER_PASSWORD,
        client,
        browser,
        asgi_live_server,
        django_user_model,
        django_username_field,
    )

    yield browser
    browser.quit()


@pytest.fixture
def preauth_asgi_browser(preauth_browser, transactional_db, asgi_live_server):  # noqa
    with wait_for_page_load(preauth_browser):
        preauth_browser.visit(asgi_live_server.url)
    wait_for_websocket_connection(preauth_browser)
    return preauth_browser


@pytest.fixture
def admin_browser(
    admin_user,
    client,
    browser,
    asgi_live_server,  # noqa
    django_user_model,
    django_username_field,
    transactional_db,
) -> DriverAPI:
    browser = _preauth_session_id_helper(
        "admin",
        "password",
        client,
        browser,
        asgi_live_server,
        django_user_model,
        django_username_field,
    )
    browser.driver.set_window_size(1920, 1600)

    yield browser
    browser.execute_script("window.onbeforeunload = function(e) {};")
    browser.quit()


@pytest.fixture(scope="function")
def uczelnia(db):
    return Uczelnia.objects.get_or_create(
        skrot="TE",
        nazwa="Testowa uczelnia",
    )[0]


@pytest.fixture
def uczelnia_z_obca_jednostka(uczelnia, obca_jednostka):
    uczelnia.obca_jednostka = obca_jednostka
    uczelnia.save()
    return uczelnia


@pytest.mark.django_db
def _wydzial_maker(nazwa, skrot, uczelnia, **kwargs):
    return Wydzial.objects.get_or_create(
        uczelnia=uczelnia, skrot=skrot, nazwa=nazwa, **kwargs
    )[0]


@pytest.mark.django_db
@pytest.fixture
def wydzial_maker(db):
    return _wydzial_maker


@pytest.mark.django_db
@pytest.fixture(scope="function")
def wydzial(uczelnia, db):
    return _wydzial_maker(uczelnia=uczelnia, skrot="W1", nazwa="Wydział Testowy I")


def _autor_maker(imiona, nazwisko, tytul="dr", **kwargs):
    tytul = Tytul.objects.get(skrot=tytul)
    return Autor.objects.get_or_create(
        tytul=tytul, imiona=imiona, nazwisko=nazwisko, **kwargs
    )[0]


@pytest.fixture
def autor_maker(db):
    return _autor_maker


@pytest.fixture(scope="function")
def autor_jan_nowak(db, tytuly):
    return _autor_maker(imiona="Jan", nazwisko="Nowak")


@pytest.fixture(scope="function")
def autor(db, tytuly):
    return baker.make(Autor)


@pytest.fixture(scope="function")
def typ_odpowiedzialnosci_autor(db):
    return Typ_Odpowiedzialnosci.objects.get_or_create(
        skrot="aut.", nazwa="autor", typ_ogolny=TO_AUTOR
    )


@pytest.fixture(scope="function")
def autor_jan_kowalski(db, tytuly):
    return _autor_maker(imiona="Jan", nazwisko="Kowalski", tytul="prof. dr hab. med.")


def _jednostka_maker(nazwa, skrot, wydzial, **kwargs):
    ret = Jednostka.objects.get_or_create(
        nazwa=nazwa, skrot=skrot, wydzial=wydzial, uczelnia=wydzial.uczelnia, **kwargs
    )[0]
    ret.refresh_from_db()
    return ret


JEDNOSTKA_UCZELNI = "Jednostka Uczelni"


@pytest.mark.django_db
@pytest.fixture(scope="function")
def jednostka(wydzial, db):

    return _jednostka_maker(JEDNOSTKA_UCZELNI, skrot="Jedn. Ucz.", wydzial=wydzial)


@pytest.mark.django_db
@pytest.fixture(scope="function")
def aktualna_jednostka(jednostka: Jednostka, wydzial, db):
    jednostka.jednostka_wydzial_set.create(wydzial=wydzial)
    jednostka.refresh_from_db()
    return jednostka


@pytest.mark.django_db
@pytest.fixture
def drugi_wydzial(uczelnia):
    return baker.make(Wydzial, uczelnia=uczelnia)


@pytest.mark.django_db
@pytest.fixture
def druga_aktualna_jednostka(druga_jednostka, drugi_wydzial):
    druga_jednostka.jednostka_wydzial_set.create(wydzial=drugi_wydzial)
    druga_jednostka.refresh_from_db()
    return druga_jednostka


JEDNOSTKA_PODRZEDNA = "Jednostka P-rzedna"


@pytest.mark.django_db
@pytest.fixture(scope="function")
def jednostka_podrzedna(jednostka):

    return _jednostka_maker(
        JEDNOSTKA_PODRZEDNA, skrot="JP", wydzial=jednostka.wydzial, parent=jednostka
    )


@pytest.mark.django_db
@pytest.fixture(scope="function")
def druga_jednostka(wydzial, db):
    return _jednostka_maker(
        "Druga Jednostka Uczelni", skrot="Dr. Jedn. Ucz.", wydzial=wydzial
    )


@pytest.mark.django_db
@pytest.fixture(scope="function")
def obca_jednostka(wydzial):
    return _jednostka_maker(
        "Obca Jednostka",
        skrot="OJ",
        wydzial=wydzial,
        skupia_pracownikow=False,
        zarzadzaj_automatycznie=False,
        widoczna=False,
        wchodzi_do_raportow=False,
    )


@pytest.fixture
def jednostka_maker():
    return _jednostka_maker


def _zrodlo_maker(nazwa, skrot, **kwargs):
    return baker.make(Zrodlo, nazwa=nazwa, skrot=skrot, **kwargs)


@pytest.fixture
def zrodlo_maker():
    return _zrodlo_maker


@pytest.fixture(scope="function")
def zrodlo(db):
    return _zrodlo_maker(nazwa="Testowe Źródło", skrot="Test. Źr.")


def set_default(varname, value, dct):
    if varname not in dct:
        dct[varname] = value


def _wydawnictwo_maker(klass, **kwargs):
    if "rok" not in kwargs:
        kwargs["rok"] = current_rok()

    c = time.time()
    kl = str(klass).split(".")[-1].replace("'>", "")

    kw_wyd = dict(
        tytul=f"Tytul {kl} {c}",
        tytul_oryginalny=f"Tytul oryginalny {kl} {c}",
        uwagi=f"Uwagi {kl} {c}",
        szczegoly=f"Szczegóły {kl} {c}",
    )

    if klass == Patent:
        del kw_wyd["tytul"]

    for key, value in kw_wyd.items():
        set_default(key, value, kwargs)

    return baker.make(klass, **kwargs)


def _wydawnictwo_ciagle_maker(**kwargs):
    if "zrodlo" not in kwargs:
        set_default(
            "zrodlo",
            _zrodlo_maker(
                nazwa="Źrodło Ciągłego Wydawnictwa", skrot="Źród. Ciąg. Wyd."
            ),
            kwargs,
        )

    set_default("informacje", "zrodlo-informacje", kwargs)
    set_default("issn", "123-IS-SN-34", kwargs)
    if "jezyk" not in kwargs:
        jezyk = Jezyk.objects.get(nazwa__icontains="polski")
        jezyk.pbn_uid = Language.objects.get_or_create(
            pk="pol",
            language={
                "de": "Polnisch",
                "en": "Polish",
                "pl": "polski",
                "639-1": "pl",
                "639-2": "pol",
                "wikiUrl": "https://en.wikipedia.org/wiki/Polish_language",
            },
        )[0]
        kwargs["jezyk"] = jezyk

    return _wydawnictwo_maker(Wydawnictwo_Ciagle, **kwargs)


def wydawnictwo_ciagle_maker(db):
    return _wydawnictwo_ciagle_maker


@pytest.fixture(scope="function")
@pytest.mark.django_db
def wydawnictwo_ciagle(
    jezyki, charaktery_formalne, typy_kbn, statusy_korekt, typy_odpowiedzialnosci
):
    ret = _wydawnictwo_ciagle_maker()
    return ret


def _zwarte_base_maker(klass, **kwargs):
    if klass not in [Praca_Doktorska, Praca_Habilitacyjna, Patent]:
        set_default("liczba_znakow_wydawniczych", 31337, kwargs)

    set_default("informacje", "zrodlo-informacje dla zwarte", kwargs)

    if klass not in [Patent]:
        set_default("miejsce_i_rok", "Lublin %s" % current_rok(), kwargs)
        set_default("wydawnictwo", "Wydawnictwo FOLIUM", kwargs)
        set_default("isbn", "123-IS-BN-34", kwargs)
        set_default("redakcja", "Redakcja", kwargs)

    return _wydawnictwo_maker(klass, **kwargs)


def _zwarte_maker(**kwargs):
    return _zwarte_base_maker(Wydawnictwo_Zwarte, **kwargs)


@pytest.fixture(scope="function")
def wydawnictwo_zwarte(
    jezyki, charaktery_formalne, typy_kbn, statusy_korekt, typy_odpowiedzialnosci
):
    """
    :rtype: bpp.models.Wydawnictwo_Zwarte
    """
    return _zwarte_maker(tytul_oryginalny="Wydawnictwo Zwarte ĄćłłóńŹ")


@pytest.fixture
def zwarte_maker(db):
    return _zwarte_maker


@pytest.fixture
def wydawca(db):
    return Wydawca.objects.get_or_create(nazwa="Wydawca Testowy")[0]


@pytest.fixture
def alias_wydawcy(wydawca):
    return Wydawca.objects.create(nazwa="Drugi taki tam", alias_dla=wydawca)


def _habilitacja_maker(**kwargs):
    Charakter_Formalny.objects.get_or_create(nazwa="Praca habilitacyjna", skrot="H")
    return _zwarte_base_maker(Praca_Habilitacyjna, **kwargs)


@pytest.fixture(scope="function")
def habilitacja(jednostka, db, charaktery_formalne, jezyki, typy_odpowiedzialnosci):
    return _habilitacja_maker(
        tytul_oryginalny="Praca habilitacyjna", jednostka=jednostka
    )


@pytest.fixture
def praca_habilitacyjna(
    habilitacja,
) -> Praca_Habilitacyjna:
    return habilitacja


@pytest.fixture
def habilitacja_maker(db):
    return _habilitacja_maker


def _doktorat_maker(**kwargs):
    return _zwarte_base_maker(Praca_Doktorska, **kwargs)


@pytest.fixture(scope="function")
def doktorat(jednostka, charaktery_formalne, jezyki, typy_odpowiedzialnosci):
    return _doktorat_maker(tytul_oryginalny="Praca doktorska", jednostka=jednostka)


@pytest.fixture(scope="function")
def praca_doktorska(doktorat):
    return doktorat


@pytest.fixture
def doktorat_maker(db):
    return _doktorat_maker


def _patent_maker(**kwargs):
    return _zwarte_base_maker(Patent, **kwargs)


@pytest.fixture
def patent(db, typy_odpowiedzialnosci, jezyki, charaktery_formalne, typy_kbn):
    return _patent_maker(tytul_oryginalny="PATENT!")


@pytest.fixture
def patent_maker(db):
    return _patent_maker


# def _lookup_fun(klass):
#     def fun(skrot):
#         return klass.objects.filter(skrot=skrot)
#     return fun
#
#
# typ_kbn = _lookup_fun(Typ_KBN)
# jezyk = _lookup_fun(Jezyk)
# charakter = _lookup_fun(Charakter_Formalny)


@pytest.fixture(scope="function")
def webtest_app(request):
    wtm = django_webtest.WebTestMixin()
    wtm._patch_settings()
    request.addfinalizer(wtm._unpatch_settings)
    return django_webtest.DjangoTestApp()


def _webtest_login(webtest_app, username, password, login_form="login_form"):
    form = webtest_app.get(reverse(login_form)).form
    form["username"] = username  # normal_django_user.username
    form["password"] = password  # NORMAL_DJANGO_USER_PASSWORD
    res = form.submit().maybe_follow()
    assert res.context["user"].username == username  # normal_django_user.username
    return webtest_app


@pytest.fixture(scope="function")
def wprowadzanie_danych_user(normal_django_user):
    from django.contrib.auth.models import Group

    # zeby bpp.core.editor_emails zwracało
    normal_django_user.email = "foo@bar.pl"

    grp = Group.objects.get_or_create(name=GR_WPROWADZANIE_DANYCH)[0]
    normal_django_user.groups.add(grp)

    normal_django_user.save()
    return normal_django_user


@pytest.fixture(scope="function")
def app(webtest_app, normal_django_user) -> webtest.app.TestApp:
    return _webtest_login(
        webtest_app, NORMAL_DJANGO_USER_LOGIN, NORMAL_DJANGO_USER_PASSWORD
    )


@pytest.fixture(scope="function")
def wd_app(webtest_app, wprowadzanie_danych_user):
    return _webtest_login(
        webtest_app, NORMAL_DJANGO_USER_LOGIN, NORMAL_DJANGO_USER_PASSWORD
    )


@pytest.fixture(scope="function")
def admin_app(webtest_app, admin_user) -> DjangoTestApp:
    """
    :rtype: django_webtest.DjangoTestApp
    """

    return _webtest_login(webtest_app, "admin", "password")


def fixture(name):
    return json.load(
        open(
            os.path.abspath(
                os.path.join(os.path.dirname(__file__), "../bpp", "fixtures", name)
            ),
            "rb",
        )
    )


@pytest.fixture(scope="function")
def typy_odpowiedzialnosci(db):
    for elem in fixture("typ_odpowiedzialnosci_v2.json"):
        Typ_Odpowiedzialnosci.objects.get_or_create(pk=elem["pk"], **elem["fields"])
    return {x.skrot: x for x in Typ_Odpowiedzialnosci.objects.all()}


@pytest.fixture(scope="function")
def tytuly():
    for elem in fixture("tytul.json"):
        Tytul.objects.get_or_create(pk=elem["pk"], **elem["fields"])


@pytest.fixture(scope="function")
def jezyki(db):
    pl, created = Jezyk.objects.get_or_create(pk=1, skrot="pol.", nazwa="polski")
    pl.skrot_dla_pbn = "PL"
    pl.save()
    assert pl.pk == 1

    ang, created = Jezyk.objects.get_or_create(pk=2, skrot="ang.", nazwa="angielski")
    ang.skrot_dla_pbn = "EN"
    ang.save()
    assert ang.pk == 2

    for elem in fixture("jezyk.json"):
        Jezyk.objects.get_or_create(**elem["fields"])

    return {jezyk.skrot: jezyk for jezyk in Jezyk.objects.all()}


@pytest.fixture(scope="function")
def charaktery_formalne():
    Charakter_Formalny.objects.all().delete()
    for elem in fixture("charakter_formalny.json"):
        Charakter_Formalny.objects.get_or_create(pk=elem["pk"], **elem["fields"])

    chf_ksp = Charakter_Formalny.objects.get(skrot="KSP")
    chf_ksp.rodzaj_pbn = const.RODZAJ_PBN_KSIAZKA
    chf_ksp.charakter_ogolny = const.CHARAKTER_OGOLNY_KSIAZKA
    chf_ksp.charakter_sloty = const.CHARAKTER_SLOTY_KSIAZKA
    chf_ksp.nazwa_w_primo = "Książka"
    chf_ksp.save()

    chf_roz = Charakter_Formalny.objects.get(skrot="ROZ")
    chf_roz.rodzaj_pbn = const.RODZAJ_PBN_ROZDZIAL
    chf_ksp.charakter_ogolny = const.CHARAKTER_OGOLNY_ROZDZIAL
    chf_roz.charakter_sloty = const.CHARAKTER_SLOTY_ROZDZIAL
    chf_roz.save()

    return {x.skrot: x for x in Charakter_Formalny.objects.all()}


@pytest.fixture(scope="function")
def ksiazka_polska(charaktery_formalne):
    return charaktery_formalne["KSP"]


@pytest.fixture(scope="function")
def artykul_w_czasopismie(charaktery_formalne):
    return charaktery_formalne["AC"]


@pytest.fixture(scope="function")
def typy_kbn():
    for elem in fixture("typ_kbn.json"):
        Typ_KBN.objects.get_or_create(pk=elem["pk"], **elem["fields"])


@pytest.fixture(scope="function")
def statusy_korekt(db):
    for elem in fixture("status_korekty.json"):
        Status_Korekty.objects.get_or_create(pk=elem["pk"], **elem["fields"])
    return {status.nazwa: status for status in Status_Korekty.objects.all()}


@pytest.fixture(scope="function")
def przed_korekta(statusy_korekt):
    return statusy_korekt["przed korektą"]


@pytest.fixture(scope="function")
def po_korekcie(statusy_korekt):
    return statusy_korekt["po korekcie"]


@pytest.fixture(scope="function")
def w_trakcie_korekty(statusy_korekt):
    return statusy_korekt["w trakcie korekty"]


@pytest.fixture(scope="function")
def funkcje_autorow():
    for elem in get_fixture("funkcja_autora").values():
        Funkcja_Autora.objects.get_or_create(**elem)


@pytest.fixture(scope="function")
def standard_data(
    typy_odpowiedzialnosci,
    tytuly,
    jezyki,
    charaktery_formalne,
    typy_kbn,
    statusy_korekt,
    funkcje_autorow,
):
    class StandardData:
        @classmethod
        def clean(self):
            for model in (
                Typ_Odpowiedzialnosci,
                Tytul,
                Jezyk,
                Charakter_Formalny,
                Typ_KBN,
                Status_Korekty,
                Funkcja_Autora,
            ):
                model.objects.all().delete()

    return StandardData


@pytest.mark.django_db
@pytest.fixture(scope="function")
def openaccess_data():
    from django.contrib.contenttypes.models import ContentType

    for model_name, skrot, nazwa in get_openaccess_data():
        klass = ContentType.objects.get_by_natural_key("bpp", model_name).model_class()
        klass.objects.get_or_create(nazwa=nazwa, skrot=skrot)


@pytest.fixture(scope="function")
def wydawnictwo_ciagle_z_autorem(
    wydawnictwo_ciagle, autor_jan_kowalski, jednostka, typy_odpowiedzialnosci
) -> Wydawnictwo_Ciagle:
    wydawnictwo_ciagle.dodaj_autora(autor_jan_kowalski, jednostka)
    return wydawnictwo_ciagle


@pytest.fixture(scope="function")
def wydawnictwo_zwarte_z_autorem(wydawnictwo_zwarte, autor_jan_kowalski, jednostka):
    wydawnictwo_zwarte.dodaj_autora(autor_jan_kowalski, jednostka)
    return wydawnictwo_zwarte


@pytest.fixture(scope="function")
def wydawnictwo_ciagle_z_dwoma_autorami(
    wydawnictwo_ciagle_z_autorem, autor_jan_nowak, jednostka, typy_odpowiedzialnosci
):
    wydawnictwo_ciagle_z_autorem.dodaj_autora(autor_jan_nowak, jednostka)
    return wydawnictwo_ciagle_z_autorem


def pytest_configure():
    from django.conf import settings

    if hasattr(settings, "RAVEN_CONFIG"):
        del settings.RAVEN_CONFIG  # setattr(settings, "RAVEN_CONFIG", None)

    settings.TESTING = True
    settings.CELERY_ALWAYS_EAGER = True
    settings.CELERY_EAGER_PROPAGATES_EXCEPTIONS = True

    from bpp.models.cache import Autorzy, Rekord

    Rekord._meta.managed = True
    Autorzy._meta.managed = True


# @pytest.fixture(scope="session")
# def splinter_driver_kwargs(splinter_webdriver):
#    if splinter_webdriver == "remote":
#        from selenium import webdriver

#        chrome_op = webdriver.ChromeOptions()
#        chrome_op.add_argument("--disable-extensions")
#        chrome_op.add_argument("--disable-extensions-file-access-check")
#        chrome_op.add_argument("--disable-extensions-http-throttling")
#        chrome_op.add_argument("--disable-infobars")
#        chrome_op.add_argument("--enable-automation")
#        chrome_op.add_argument("--start-maximized")
#        chrome_op.add_experimental_option('prefs', {
#            'credentials_enable_service': False,
#            'profile': {
#                'password_manager_enabled': False
#            }
#        })
#        return {'browser': "chrome",
#                "desired_capabilities": chrome_op.to_capabilities()}

collect_ignore = [os.path.join(os.path.dirname(__file__), "media")]

import os

import pytest


@pytest.fixture
def denorms():
    from denorm import denorms

    yield denorms


@pytest.fixture
def praca_z_dyscyplina(wydawnictwo_ciagle_z_autorem, dyscyplina1, rok, db, denorms):

    wydawnictwo_ciagle_z_autorem.punkty_kbn = 5
    wydawnictwo_ciagle_z_autorem.save()

    wca = wydawnictwo_ciagle_z_autorem.autorzy_set.first()
    Autor_Dyscyplina.objects.create(
        autor=wca.autor, rok=wca.rekord.rok, dyscyplina_naukowa=dyscyplina1
    )
    wca.dyscyplina_naukowa = dyscyplina1
    wca.save()

    denorms.flush()

    return wydawnictwo_ciagle_z_autorem


@pytest.fixture
def api_client(client):
    return APIClient()


@pytest.fixture
def baza_wos():
    return Zewnetrzna_Baza_Danych.objects.get_or_create(
        nazwa="Web of Science", skrot="WOS"
    )[0]


from asgi_live_server import asgi_live_server  # noqa


@pytest.fixture
def wydawnictwo_zwarte_przed_korekta(statusy_korekt):
    return baker.make(
        Wydawnictwo_Zwarte, status_korekty=statusy_korekt["przed korektą"]
    )


@pytest.fixture
def wydawnictwo_zwarte_w_trakcie_korekty(statusy_korekt):
    return baker.make(
        Wydawnictwo_Zwarte, status_korekty=statusy_korekt["w trakcie korekty"]
    )


@pytest.fixture
def wydawnictwo_zwarte_po_korekcie(statusy_korekt):
    return baker.make(Wydawnictwo_Zwarte, status_korekty=statusy_korekt["po korekcie"])


@pytest.fixture
def ksiazka(wydawnictwo_zwarte, ksiazka_polska) -> "Wydawnictwo_Zwarte":
    wydawnictwo_zwarte.charakter_formalny = ksiazka_polska
    wydawnictwo_zwarte.save()
    return wydawnictwo_zwarte


@pytest.fixture
def artykul(wydawnictwo_ciagle, artykul_w_czasopismie):
    wydawnictwo_ciagle.charakter_formalny = artykul_w_czasopismie
    wydawnictwo_ciagle.save()
    return wydawnictwo_ciagle


def gen_kod_dyscypliny_func():
    top = random.randint(1, 8)
    bottom = random.randint(1, 500)
    return f"{top}.{bottom}"


baker.generators.add(
    "bpp.models.dyscyplina_naukowa.KodDyscyplinyField", gen_kod_dyscypliny_func
)


@pytest.fixture
def autor_z_dyscyplina(autor_jan_nowak, dyscyplina1, rok) -> Autor_Dyscyplina:
    return Autor_Dyscyplina.objects.get_or_create(
        autor=autor_jan_nowak, dyscyplina_naukowa=dyscyplina1, rok=rok
    )[0]


#
# Monkeypatch fixture-teardown to allow TRUNCATE
#


from django.core.management import call_command
from django.db import connections
from django.test import TransactionTestCase


def _fixture_teardown(self):
    # Allow TRUNCATE ... CASCADE and don't emit the post_migrate signal
    # when flushing only a subset of the apps
    for db_name in self._databases_names(include_mirrors=False):
        # Flush the database
        inhibit_post_migrate = (
            self.available_apps is not None
            or (  # Inhibit the post_migrate signal when using serialized
                # rollback to avoid trying to recreate the serialized data.
                self.serialized_rollback
                and hasattr(connections[db_name], "_test_serialized_contents")
            )
        )
        call_command(
            "flush",
            verbosity=0,
            interactive=False,
            database=db_name,
            reset_sequences=False,
            # In the real TransactionTestCase this is conditionally set to False.
            allow_cascade=True,
            inhibit_post_migrate=inhibit_post_migrate,
        )


TransactionTestCase._fixture_teardown = _fixture_teardown


def pytest_collection_modifyitems(items):
    # Dodaj marker "selenium" dla wszystkich testów uzywających fikstur 'browser'
    # lub 'admin_browser', aby można było szybko uruchamiać wyłacznie te testy
    # lub nie uruchamiać ich:

    flaky_test = pytest.mark.flaky(reruns=10)

    for item in items:
        fixtures = getattr(item, "fixturenames", ())
        if "browser" in fixtures or "admin_browser" in fixtures:
            item.add_marker("selenium")
            item.add_marker(flaky_test)


@pytest.fixture
def szablony():
    dirname = os.path.dirname(__file__)

    def template_n(elem):
        return f"{dirname}/../bpp/templates/{elem}"

    def create_template(Template, name):
        Template.objects.create(
            name=name,
            content=open(template_n(name)).read(),
        )

    def instaluj_szablony():

        create_template(Template, "opis_bibliograficzny.html")
        create_template(Template, "browse/praca_tabela.html")

        SzablonDlaOpisuBibliograficznego.objects.create(
            model=None,
            template=Template.objects.get(name="opis_bibliograficzny.html"),
        )

    instaluj_szablony()
    return Template.objects


import pytest


@pytest.fixture(scope="session")
def django_db_setup(django_db_setup, django_db_blocker):
    from denorm import denorms

    with django_db_blocker.unblock():
        denorms.install_triggers()


@pytest.fixture(scope="function")
def django_db_setup(django_db_setup, django_db_blocker):  # noqa
    from denorm import denorms

    with django_db_blocker.unblock():
        denorms.install_triggers()


@pytest.fixture(scope="class")
def django_db_setup(django_db_setup, django_db_blocker):  # noqa
    from denorm import denorms

    with django_db_blocker.unblock():
        denorms.install_triggers()


# https://github.com/pytest-dev/pytest-splinter/issues/158
#  AttributeError: module 'splinter.driver.webdriver.firefox' has no attribute 'WebDriverElement'


from pytest_splinter.webdriver_patches import patch_webdriver


@pytest.fixture(scope="session")
def browser_patches():
    patch_webdriver()



# Generated by Django 3.0.9 on 2020-09-30 21:49

import django.contrib.postgres.fields.jsonb
import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="FormRepresentation",
            fields=[
                (
                    "full_name",
                    models.TextField(
                        primary_key=True, serialize=False, verbose_name="Kod formularza"
                    ),
                ),
                ("label", models.TextField(verbose_name="Nazwa formularza")),
                (
                    "html_before",
                    models.TextField(
                        blank=True, null=True, verbose_name="Kod HTML przed formularzem"
                    ),
                ),
                (
                    "html_after",
                    models.TextField(
                        blank=True, null=True, verbose_name="Kod HTML po formularzu"
                    ),
                ),
            ],
            options={
                "verbose_name": "Lista wartości domyślnych formularza",
                "verbose_name_plural": "Listy wartości domyślnych formularzy",
                "unique_together": {("full_name", "label")},
            },
        ),
        migrations.CreateModel(
            name="FormFieldRepresentation",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.TextField(verbose_name="Systemowa nazwa pola")),
                (
                    "label",
                    models.TextField(
                        blank=True, null=True, verbose_name="Czytelna etykieta pola"
                    ),
                ),
                ("klass", models.TextField(verbose_name="Klasa pola")),
                ("order", models.PositiveSmallIntegerField()),
                (
                    "parent",
                    models.ForeignKey(
                        help_text="Formularz, do którego należy to pole",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="fields_set",
                        to="formdefaults.FormRepresentation",
                    ),
                ),
            ],
            options={
                "verbose_name": "Pole formularza",
                "verbose_name_plural": "Pola formularzy",
                "ordering": ("order",),
                "unique_together": {("parent", "name")},
            },
        ),
        migrations.CreateModel(
            name="FormFieldDefaultValue",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "value",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        blank=True, null=True, verbose_name="Wartość"
                    ),
                ),
                (
                    "field",
                    models.ForeignKey(
                        help_text="Pole formularza, dla którego zdefiniowana jest ta wartość domyślna. ",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="formdefaults.FormFieldRepresentation",
                        verbose_name="Pole formularza",
                    ),
                ),
                (
                    "parent",
                    models.ForeignKey(
                        help_text="Formularz, do którego należy pole, dla którego to zdefiniowana jestta wartość domyślna pola. ",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="values_set",
                        to="formdefaults.FormRepresentation",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="Użytkownik",
                    ),
                ),
            ],
            options={
                "verbose_name": "Wartość domyslna dla pola formularza",
                "verbose_name_plural": "Wartości domyślne dla pól formularzy",
                "ordering": ("user", "field__order"),
            },
        ),
    ]



# Generated by Django 3.2.14 on 2022-07-07 22:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("import_list_if", "0002_auto_20210308_1246"),
    ]

    operations = [
        migrations.AlterField(
            model_name="importlistifrow",
            name="dane_z_xls",
            field=models.JSONField(),
        ),
    ]



# -*- encoding: utf-8 -*-

from datetime import timedelta

import pytest

from django.utils import timezone

from bpp.models.autor import Autor_Jednostka


@pytest.mark.django_db
def test_0046_ostatnia_jednostka_trigger_via_api(
    autor_jan_nowak, jednostka, standard_data
):
    jednostka.dodaj_autora(autor_jan_nowak)
    assert autor_jan_nowak.aktualna_jednostka == jednostka


@pytest.mark.django_db
def test_0046_ostatnia_jednostka_trigger(
    autor_jan_nowak, jednostka, druga_jednostka, standard_data
):
    assert autor_jan_nowak.aktualna_jednostka is None

    two_months = timedelta(days=60)
    month = timedelta(days=30)

    # Strategia 3 test 1
    aj = Autor_Jednostka.objects.create(
        autor=autor_jan_nowak,
        jednostka=jednostka,
        rozpoczal_prace=timezone.now() - two_months,
    )
    autor_jan_nowak.refresh_from_db()
    assert autor_jan_nowak.aktualna_jednostka == jednostka

    aj.zakonczyl_prace = timezone.now() - month
    aj.save()

    autor_jan_nowak.refresh_from_db()
    assert autor_jan_nowak.aktualna_jednostka is None

    aj.delete()
    autor_jan_nowak.refresh_from_db()
    assert autor_jan_nowak.aktualna_jednostka is None

    # Strategia 3 test 2
    aj = Autor_Jednostka.objects.create(
        autor=autor_jan_nowak,
        jednostka=jednostka,
    )
    autor_jan_nowak.refresh_from_db()
    assert autor_jan_nowak.aktualna_jednostka == jednostka

    aj.delete()
    autor_jan_nowak.refresh_from_db()
    assert autor_jan_nowak.aktualna_jednostka is None

    # Strategia 1
    aj = Autor_Jednostka.objects.create(
        autor=autor_jan_nowak,
        jednostka=jednostka,
        rozpoczal_prace=timezone.now() - two_months,
    )
    aj2 = Autor_Jednostka.objects.create(
        autor=autor_jan_nowak,
        jednostka=druga_jednostka,
        rozpoczal_prace=timezone.now() - two_months,
        zakonczyl_prace=timezone.now() - month,
    )
    autor_jan_nowak.refresh_from_db()
    assert autor_jan_nowak.aktualna_jednostka == jednostka

    aj.delete()
    aj2.delete()
    autor_jan_nowak.refresh_from_db()
    assert autor_jan_nowak.aktualna_jednostka is None

    Autor_Jednostka.objects.all().delete()

    # Strategia 2
    aj = Autor_Jednostka.objects.create(
        autor=autor_jan_nowak,
        jednostka=jednostka,
        rozpoczal_prace=timezone.now() - two_months,
    )
    aj2 = Autor_Jednostka.objects.create(
        autor=autor_jan_nowak,
        jednostka=druga_jednostka,
        rozpoczal_prace=timezone.now() - month,
    )
    autor_jan_nowak.refresh_from_db()
    assert autor_jan_nowak.aktualna_jednostka == druga_jednostka

    aj.delete()
    aj2.delete()
    autor_jan_nowak.refresh_from_db()
    assert autor_jan_nowak.aktualna_jednostka is None



from typing import List, Tuple

from bpp.models import Autor_Dyscyplina, Cache_Punktacja_Autora_Query_View


def _get_kwargs(od_roku, do_roku, prefix=""):
    kwargs = dict()
    if od_roku:
        kwargs[f"{prefix}rok__gte"] = od_roku

    if do_roku:
        kwargs[f"{prefix}rok__lte"] = do_roku
    return kwargs


def autorzy_z_dyscyplinami(od_roku=None, do_roku=None) -> List[Tuple[int, int, int]]:
    """
    Zwraca listę autorów z dyscyplinami z bazy danych

    :returns: listę z krotkami z których każda krotka ma następujące wartości:
        ID autora, rok, ID dyscypliny naukowej
    """

    kwargs = _get_kwargs(od_roku, do_roku)

    return (
        Autor_Dyscyplina.objects.values("autor_id", "rok", "dyscyplina_naukowa_id")
        .filter(**kwargs)
        .exclude(dyscyplina_naukowa_id=None)
        .union(
            Autor_Dyscyplina.objects.values(
                "autor_id", "rok", "subdyscyplina_naukowa_id"
            )
            .exclude(subdyscyplina_naukowa_id=None)
            .filter(**kwargs)
        )
    )


def autorzy_z_punktami(
    od_roku=None, do_roku=None, min_pk=None
) -> List[Tuple[int, int, int]]:
    """
    Zwraca listę autorów z dyscyplinami z punktami z bazy danych

    """

    kwargs = _get_kwargs(od_roku, do_roku, prefix="rekord__")

    exclude_kwargs = dict()
    if min_pk is not None:
        exclude_kwargs = dict(rekord__punkty_kbn__lte=min_pk)

    return (
        Cache_Punktacja_Autora_Query_View.objects.all()
        .filter(**kwargs)
        .exclude(**exclude_kwargs)
        .values("autor_id", "rekord__rok", "dyscyplina_id")
    )


def autorzy_zerowi(od_roku=None, do_roku=None, min_pk=None):
    """
    Zwraca listę krotek w postaci (autor_id, rok, dyscyplina_id) która zawiera listę
    autorów zerowych czyli autorów, którzy mimo zadeklarowanych na dany rok dyscyplin
    nie posiadają w bazie żadnych punktowanych rekordów.
    """
    # wartośći zadeklarowane w bazie danych
    defined = autorzy_z_dyscyplinami(od_roku=od_roku, do_roku=do_roku)

    # zestawy autor/rok/dyscyplina z całej bazy danych
    existent = autorzy_z_punktami(od_roku=od_roku, do_roku=do_roku, min_pk=min_pk)

    return defined.difference(existent)



from collections import namedtuple
from datetime import datetime

from django.db.models import F, Transform

from ewaluacja2021 import const

from django.contrib.postgres.aggregates import ArrayAgg

from bpp.const import RODZAJ_PBN_ARTYKUL
from bpp.util import intsack


class NieArtykul(Transform):

    template = f"(%(expressions)s != {RODZAJ_PBN_ARTYKUL})"


def get_lista_prac(nazwa_dyscypliny):
    """Zwraca liste prac - potencjalnych kandydatow do ewaluacji, ale wyłcznie dla dozwolonych
    autorow, tzn posiadajacych udzialy jednostkowe w danej dyscyplinie oraz dla lat 2018-2021
    """
    from ewaluacja2021.models import IloscUdzialowDlaAutora

    dozwoleni_autorzy = IloscUdzialowDlaAutora.objects.filter(
        dyscyplina_naukowa__nazwa=nazwa_dyscypliny
    ).values_list("autor_id")

    if not dozwoleni_autorzy.exists():
        raise ValueError(
            f"Nie mam żadnych autorów z wgranymi udziałami dla dyscypliny {nazwa_dyscypliny}"
        )

    from bpp.models import Cache_Punktacja_Autora_Query

    return (
        Cache_Punktacja_Autora_Query.objects.filter(
            rekord__rok__gte=const.ROK_MIN,
            rekord__rok__lte=const.ROK_MAX,
            dyscyplina__nazwa=nazwa_dyscypliny,
            autor__in=dozwoleni_autorzy,
        )
        .exclude(rekord__charakter_formalny__charakter_ogolny=None)
        .annotate(
            monografia=NieArtykul(F("rekord__charakter_formalny__rodzaj_pbn")),
            rok=F("rekord__rok"),
        )
        .select_related(
            "rekord",
            "rekord__charakter_formalny",
        )
    )


def get_lista_autorow_na_rekord(nazwa_dyscypliny):
    return {
        x["rekord_id"]: tuple(x["autorzy"])
        for x in get_lista_prac(nazwa_dyscypliny)
        .values("rekord_id")
        .annotate(autorzy=ArrayAgg("autor_id"))
        .order_by()
    }


def lista_prac_na_tuples(lista_prac, lista_autorow):
    return tuple(
        Praca(
            id=elem.id,
            rekord_id=elem.rekord_id,
            slot=elem.slot,
            autor_id=elem.autor_id,
            rok=elem.rekord.rok,
            pkdaut=elem.pkdaut,
            monografia=elem.monografia,
            autorzy=lista_autorow.get(elem.rekord_id),
            ostatnio_zmieniony=elem.rekord.ostatnio_zmieniony,
        )
        for elem in lista_prac
    )


def get_lista_prac_as_tuples(nazwa_dyscypliny):
    return lista_prac_na_tuples(
        list(get_lista_prac(nazwa_dyscypliny)),
        get_lista_autorow_na_rekord(nazwa_dyscypliny),
    )


def policz_knapsack(lista_prac, maks_slot=4.0):
    res = intsack(
        maks_slot,
        [x.slot for x in lista_prac],
        [x.pkdaut for x in lista_prac],
        [x for x in lista_prac],
    )
    return res


def encode_datetime(obj):
    if isinstance(obj, datetime):
        return obj.isoformat()
    raise TypeError(
        "Object of type %s is not JSON serializable" % obj.__class__.__name__
    )


def maks_pkt_aut_calosc_get_from_db(nazwa_dyscypliny):
    from ewaluacja2021.models import IloscUdzialowDlaAutora

    return {
        int(x["autor_id"]): x["ilosc_udzialow"]
        for x in IloscUdzialowDlaAutora.objects.filter(
            dyscyplina_naukowa__nazwa=nazwa_dyscypliny
        ).values("autor_id", "ilosc_udzialow")
    }


def maks_pkt_aut_monografie_get_from_db(nazwa_dyscypliny):
    from ewaluacja2021.models import IloscUdzialowDlaAutora

    return {
        int(x["autor_id"]): x["ilosc_udzialow_monografie"]
        for x in IloscUdzialowDlaAutora.objects.filter(
            dyscyplina_naukowa__nazwa=nazwa_dyscypliny
        ).values("autor_id", "ilosc_udzialow_monografie")
    }


def splitEveryN(n, it):
    return [it[i : i + n] for i in range(0, len(it), n)]


Praca = namedtuple(
    "Praca",
    [
        "id",
        "rekord_id",
        "slot",
        "autor_id",
        "rok",
        "pkdaut",
        "monografia",
        "autorzy",
        "ostatnio_zmieniony",
    ],
)



from django import forms
from taggit.forms import TextareaTagWidget

from .core import generuj_inline_dla_autorow
from .element_repozytorium import Element_RepozytoriumInline
from .filters import OstatnioZmienionePrzezFilter, UtworzonePrzezFilter
from .grant import Grant_RekorduInline
from .helpers import (
    ADNOTACJE_Z_DATAMI_FIELDSET,
    MODEL_OPCJONALNIE_NIE_EKSPORTOWANY_DO_API_FIELDSET,
    MODEL_PUNKTOWANY_FIELDSET,
    MODEL_PUNKTOWANY_KOMISJA_CENTRALNA_FIELDSET,
    MODEL_Z_ROKIEM,
    MODEL_Z_WWW,
    MODEL_ZE_SZCZEGOLAMI,
    POZOSTALE_MODELE_FIELDSET,
    AdnotacjeZDatamiMixin,
    DomyslnyStatusKorektyMixin,
    Wycinaj_W_z_InformacjiMixin,
)
from .wydawnictwo_zwarte import Wydawnictwo_ZwarteAdmin_Baza

from django.contrib import admin

from bpp.models.patent import Patent, Patent_Autor


class Patent_Form(Wycinaj_W_z_InformacjiMixin, forms.ModelForm):
    status_korekty = DomyslnyStatusKorektyMixin.status_korekty

    class Meta:
        fields = "__all__"

        widgets = {
            "slowa_kluczowe": TextareaTagWidget(attrs={"rows": 2}),
        }


class Patent_Admin(AdnotacjeZDatamiMixin, Wydawnictwo_ZwarteAdmin_Baza):
    inlines = (
        generuj_inline_dla_autorow(Patent_Autor),
        Grant_RekorduInline,
        Element_RepozytoriumInline,
    )

    list_display = ["tytul_oryginalny", "ostatnio_zmieniony"]

    search_fields = [
        "tytul_oryginalny",
        "szczegoly",
        "uwagi",
        "informacje",
        "slowa_kluczowe__name",
        "rok",
        "adnotacje",
        "id",
    ]

    list_filter = [
        "status_korekty",
        "recenzowana",
        OstatnioZmienionePrzezFilter,
        UtworzonePrzezFilter,
    ]

    form = Patent_Form

    fieldsets = (
        (
            "Patent",
            {
                "fields": ("tytul_oryginalny",)
                + MODEL_ZE_SZCZEGOLAMI
                + (
                    "wydzial",
                    "rodzaj_prawa",
                    "data_zgloszenia",
                    "numer_zgloszenia",
                    "data_decyzji",
                    "numer_prawa_wylacznego",
                    "wdrozenie",
                )
                + MODEL_Z_ROKIEM
                + MODEL_Z_WWW
            },
        ),
        MODEL_PUNKTOWANY_FIELDSET,
        MODEL_PUNKTOWANY_KOMISJA_CENTRALNA_FIELDSET,
        POZOSTALE_MODELE_FIELDSET,
        ADNOTACJE_Z_DATAMI_FIELDSET,
        MODEL_OPCJONALNIE_NIE_EKSPORTOWANY_DO_API_FIELDSET,
    )


admin.site.register(Patent, Patent_Admin)



# -*- coding: utf-8 -*-


from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0020_auto_20150824_1609'),
    ]

    operations = [
        migrations.AddField(
            model_name='jezyk',
            name='skrot_dla_pbn',
            field=models.CharField(help_text=b'\n    Skr\xc3\xb3t nazwy j\xc4\x99zyka u\xc5\xbcywany w plikach eksportu do PBN.', max_length=10, verbose_name=b'Skr\xc3\xb3t dla PBN', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='zrodlo',
            name='jezyk',
            field=models.ForeignKey(on_delete=models.CASCADE, blank=True, to='bpp.Jezyk', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='zrodlo',
            name='wydawca',
            field=models.CharField(max_length=250, blank=True),
            preserve_default=True,
        ),
    ]



# Generated by Django 3.0.7 on 2020-07-27 22:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0219_auto_20200727_2308"),
    ]

    operations = [
        migrations.AlterField(
            model_name="grant",
            name="numer_projektu",
            field=models.CharField(max_length=200, unique=True),
        ),
    ]



# Register your models here.
from rozbieznosci_if.models import IgnorujRozbieznoscIf

from django.contrib import admin


@admin.register(IgnorujRozbieznoscIf)
class IgnorujRozbieznoscIfAdmin(admin.ModelAdmin):
    list_display = ["object", "created_on"]



from .base import BasePBNMongoDBModel


class Conference(BasePBNMongoDBModel):
    class Meta:
        verbose_name = "Konferencja w PBN API"
        verbose_name_plural = "Konferencje w PBN API"

    def fullName(self):
        return self.value("object", "fullName")

    def startDate(self):
        return self.value("object", "startDate")

    def endDate(self):
        return self.value("object", "endDate")

    def city(self):
        return self.value("object", "city")

    def country(self):
        return self.value("object", "country")

    def __str__(self):
        return f"{self.fullName()}, {self.startDate()}, {self.city()}"



# Generated by Django 2.2.8 on 2020-02-13 20:47

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ("celeryui", "0003_uuid_field"),
    ]

    operations = [
        migrations.AlterField(
            model_name="report",
            name="uid",
            field=models.UUIDField(
                blank=True, default=uuid.uuid4, editable=False, unique=True
            ),
        ),
    ]



# -*- encoding: utf-8 -*-

from django import forms
from django.contrib import admin
from django.forms.widgets import Textarea

from .models import Article

SmallerTextarea = Textarea(attrs={'cols': 75, 'rows': 2})
BiggerTextarea = Textarea(attrs={'cols': 75, 'rows': 18})


class ArticleForm(forms.ModelForm):
    class Meta:
        fields = [
            'title',
            'article_body',
            'status',
            'published_on',
            'slug'
        ]
        model = Article
        widgets = {
            'title': SmallerTextarea,
            'article_body': BiggerTextarea
        }


@admin.register(Article)
class ArticleAdmin(admin.ModelAdmin):
    search_fields = ['title', 'article_body']
    list_display = ['title', 'status', 'created', 'published_on']
    form = ArticleForm
    prepopulated_fields = {"slug": ("title",)}



# Generated by Django 3.0.11 on 2021-02-23 21:28

import taggit.managers
from django.db import migrations

from bpp.migration_util import load_custom_sql


class Migration(migrations.Migration):

    dependencies = [
        ("taggit", "0003_taggeditem_add_unique_index"),
        ("bpp", "0236_merge_20210202_0850"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="patent",
            name="slowa_kluczowe",
        ),
        migrations.AddField(
            model_name="patent",
            name="slowa_kluczowe",
            field=taggit.managers.TaggableManager(
                help_text="A comma-separated list of tags.",
                through="taggit.TaggedItem",
                to="taggit.Tag",
                verbose_name="Słowa kluczowe",
            ),
        ),
        migrations.RemoveField(
            model_name="praca_doktorska",
            name="slowa_kluczowe",
        ),
        migrations.AddField(
            model_name="praca_doktorska",
            name="slowa_kluczowe",
            field=taggit.managers.TaggableManager(
                help_text="A comma-separated list of tags.",
                through="taggit.TaggedItem",
                to="taggit.Tag",
                verbose_name="Słowa kluczowe",
            ),
        ),
        migrations.RemoveField(
            model_name="praca_habilitacyjna",
            name="slowa_kluczowe",
        ),
        migrations.AddField(
            model_name="praca_habilitacyjna",
            name="slowa_kluczowe",
            field=taggit.managers.TaggableManager(
                help_text="A comma-separated list of tags.",
                through="taggit.TaggedItem",
                to="taggit.Tag",
                verbose_name="Słowa kluczowe",
            ),
        ),
        migrations.RemoveField(
            model_name="wydawnictwo_ciagle",
            name="slowa_kluczowe",
        ),
        migrations.AddField(
            model_name="wydawnictwo_ciagle",
            name="slowa_kluczowe",
            field=taggit.managers.TaggableManager(
                help_text="A comma-separated list of tags.",
                through="taggit.TaggedItem",
                to="taggit.Tag",
                verbose_name="Słowa kluczowe",
            ),
        ),
        migrations.RemoveField(
            model_name="wydawnictwo_zwarte",
            name="slowa_kluczowe",
        ),
        migrations.AddField(
            model_name="wydawnictwo_zwarte",
            name="slowa_kluczowe",
            field=taggit.managers.TaggableManager(
                help_text="A comma-separated list of tags.",
                through="taggit.TaggedItem",
                to="taggit.Tag",
                verbose_name="Słowa kluczowe",
            ),
        ),
        #
        # Powyższe operacje wywołają skasowanie tabeli bpp_rekord_mat
        #
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql("0237_rekord_mat_slowa_kluczowe")
        ),
        # Odbudowanie tabeli rekord_mat spowoduje skasowanie bpp_uczelnia_ewaluacja_view,
        # stąd potrzeba zaciągnąć ten plik z poprzednich migracji:
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql("0207_uczelnia_analiza_view")
        ),
    ]



import xlrd
from django.forms import DateField


class ExcelDateField(DateField):
    def to_python(self, value):
        if isinstance(value, float):
            return xlrd.xldate_as_datetime(value, 0).date()
        return super(ExcelDateField, self).to_python(value)



"""Ten moduł zawiera 'normalne', dla ludzi funkcje, które mogą być używane
do ustawiania testów."""
from __future__ import annotations

import cgi
import random
import re
import time
import warnings
from datetime import datetime

from django.http import HttpResponse
from django.urls import reverse
from model_bakery import baker
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.expected_conditions import visibility_of
from selenium.webdriver.support.wait import WebDriverWait
from splinter.exceptions import ElementDoesNotExist

from bpp.models import (
    Autor,
    Charakter_Formalny,
    Jednostka,
    Jezyk,
    Patent,
    Praca_Doktorska,
    Praca_Habilitacyjna,
    Typ_KBN,
    Tytul,
    Uczelnia,
    Wydawnictwo_Ciagle,
    Wydawnictwo_Zwarte,
    Wydzial,
    Zrodlo,
)
from bpp.models.system import Status_Korekty

from django_bpp.selenium_util import (
    LONG_WAIT_TIME,
    SHORT_WAIT_TIME,
    wait_for,
    wait_for_page_load,
)


def setup_model_bakery():
    baker.generators.add(
        "django.contrib.postgres.fields.array.ArrayField", lambda x: []
    )

    baker.generators.add(
        "django.contrib.postgres.search.SearchVectorField", lambda x=None: None
    )


def set_default(varname, value, dct):
    if varname not in dct:
        dct[varname] = value


def any_autor(nazwisko="Kowalski", imiona="Jan Maria", tytul="dr", **kw):
    tytul = Tytul.objects.get_or_create(skrot=tytul)[0]
    return Autor.objects.create(nazwisko=nazwisko, tytul=tytul, imiona=imiona, **kw)


def any_uczelnia(nazwa="Uczelnia", skrot="UCL"):
    return Uczelnia.objects.create(nazwa=nazwa, skrot=skrot)


wydzial_cnt = 0


def any_wydzial(nazwa=None, skrot=None, uczelnia_skrot="UCL", **kw):
    global wydzial_cnt
    try:
        uczelnia = Uczelnia.objects.get(skrot=uczelnia_skrot)
    except Uczelnia.DoesNotExist:
        uczelnia = any_uczelnia()

    if nazwa is None:
        nazwa = "Wydział %s" % wydzial_cnt

    if skrot is None:
        skrot = "W%s" % wydzial_cnt

    wydzial_cnt += 1

    set_default("uczelnia", uczelnia, kw)
    return Wydzial.objects.create(nazwa=nazwa, skrot=skrot, **kw)


def any_jednostka(nazwa=None, skrot=None, wydzial_skrot="WDZ", **kw):
    """
    :rtype: bpp.models.Jednostka
    """
    if nazwa is None:
        nazwa = "Jednostka %s" % random.randint(0, 500000)

    if skrot is None:
        skrot = "J. %s" % random.randint(0, 5000000)

    try:
        uczelnia = kw.pop("uczelnia")
    except KeyError:
        uczelnia = Uczelnia.objects.all().first()
        if uczelnia is None:
            uczelnia = baker.make(Uczelnia)

    try:
        wydzial = kw.pop("wydzial")
    except KeyError:
        try:
            wydzial = Wydzial.objects.get(skrot=wydzial_skrot)
        except Wydzial.DoesNotExist:
            wydzial = baker.make(Wydzial, uczelnia=uczelnia)

    ret = Jednostka.objects.create(
        nazwa=nazwa, skrot=skrot, wydzial=wydzial, uczelnia=uczelnia, **kw
    )
    ret.refresh_from_db()
    return ret


CURRENT_YEAR = datetime.now().year


def any_wydawnictwo(klass, rok=None, **kw):
    if rok is None:
        rok = CURRENT_YEAR

    c = time.time()
    kl = str(klass).split(".")[-1].replace("'>", "")

    kw_wyd = dict(
        tytul=f"Tytul {kl} {c}",
        tytul_oryginalny=f"Tytul oryginalny {kl} {c}",
        uwagi=f"Uwagi {kl} {c}",
        szczegoly=f"Szczegóły {kl} {c}",
    )

    if klass == Patent:
        del kw_wyd["tytul"]

    for key, value in list(kw_wyd.items()):
        set_default(key, value, kw)

    Status_Korekty.objects.get_or_create(pk=1, nazwa="przed korektą")

    return baker.make(klass, rok=rok, **kw)


def any_ciagle(**kw):
    """
    :rtype: bpp.models.Wydawnictwo_Ciagle
    """
    if "zrodlo" not in kw:
        set_default("zrodlo", any_zrodlo(), kw)
    set_default("informacje", "zrodlo-informacje", kw)
    set_default("issn", "123-IS-SN-34", kw)
    return any_wydawnictwo(Wydawnictwo_Ciagle, **kw)


def any_zwarte_base(klass, **kw):
    if klass not in [Praca_Doktorska, Praca_Habilitacyjna, Patent]:
        set_default("liczba_znakow_wydawniczych", 31337, kw)

    set_default("informacje", "zrodlo-informacje dla zwarte", kw)

    if klass not in [Patent]:
        set_default("miejsce_i_rok", "Lublin %s" % CURRENT_YEAR, kw)
        set_default("wydawca_opis", "Wydawnictwo FOLIUM", kw)
        set_default("isbn", "123-IS-BN-34", kw)
        set_default("redakcja", "Redakcja", kw)

    return any_wydawnictwo(klass, **kw)


def any_zwarte(**kw):
    """
    :rtype: bpp.models.Wydawnictwo_Zwarte
    """
    return any_zwarte_base(Wydawnictwo_Zwarte, **kw)


def any_habilitacja(**kw):
    """
    :rtype: bpp.models.Praca_Habilitacyjna
    """
    if "jednostka" not in kw:
        kw["jednostka"] = any_jednostka()
    return any_zwarte_base(Praca_Habilitacyjna, **kw)


def any_doktorat(**kw):
    """
    :rtype: bpp.models.Praca_Habilitacyjna
    """
    if "jednostka" not in kw:
        kw["jednostka"] = any_jednostka()
    return any_zwarte_base(Praca_Doktorska, **kw)


def any_patent(**kw):
    """
    :rtype: bpp.models.Patent
    """
    return any_zwarte_base(Patent, **kw)


def any_zrodlo(**kw):
    """
    :rtype: bpp.models.Zrodlo
    """
    if "nazwa" not in kw:
        kw["nazwa"] = "Zrodlo %s" % time.time()

    if "skrot" not in kw:
        kw["skrot"] = "Zrod. %s" % time.time()

    return baker.make(Zrodlo, **kw)


def _lookup_fun(klass):
    def fun(skrot):
        return klass.objects.filter(skrot=skrot)

    return fun


typ_kbn = _lookup_fun(Typ_KBN)
jezyk = _lookup_fun(Jezyk)
charakter = _lookup_fun(Charakter_Formalny)


def scroll_into_view(browser, arg):
    warnings.warn("Uzyj show_element zamiast", DeprecationWarning, stacklevel=2)
    s = f"document.getElementById('{arg}').scrollIntoView(); window.scrollBy(0,-100);"
    return browser.execute_script(s)


def show_element(browser, element):
    s = """
        // console.log('enter---');
        window.scrollTo(0, 0);
        var viewPortHeight = Math.max(document.documentElement.clientHeight, window.innerHeight || 0);
        // console.log(viewPortHeight);
        var elementTop = arguments[0].getBoundingClientRect().top;
        // console.log(elementTop);
        if (elementTop < (viewPortHeight/2)*0.5 || elementTop > (viewPortHeight/2)*1.5 ) {
            // console.log("scrolling");
            window.scrollTo(0, Math.max(0, elementTop-(viewPortHeight/2)));
            // console.log(Math.max(0, elementTop-(viewPortHeight/2)));
        }
        """
    return browser.execute_script(s, element._element)


def select_select2_autocomplete(
    browser, element_id, value, wait_for_new_value=True, value_before_enter=None
):
    """
    Wypełnia kontrolkę Select2

    :param browser: splinter.Browser
    :param element_id: ID elementu (tekst)
    :param value: tekst do wpisania
    """

    old_value = browser.find_by_id(f"select2-{element_id}-container").text

    # Znajdź interesujący nas select2-autocomplete
    element = browser.find_by_id(element_id)[0]
    sibling = element.find_by_xpath("following-sibling::span")

    if len(sibling) == 0:
        raise ElementDoesNotExist("sibling not found")

    sibling = sibling.first

    # Umieść go na widoku
    show_element(browser, sibling)

    # Kliknij w aktywny element, następnie wyślij klawisze do aktywnego
    # elementu, który się pojawił (wyskakujący pop-up select2)
    # następnie wyślij ENTER, następnie sprawdź, czy ustawiona została
    # nowa wartość. Jeżeli nie, to powtórz, maksimum 3 razy:

    # tries = 0
    # while True:

    sibling.click()

    # W tym momencie pojawi sie popup. Input type z klasą .select2-search__field
    # będzie jedyny na stronie:
    WebDriverWait(browser, SHORT_WAIT_TIME).until(
        lambda driver: driver.find_by_css(".select2-search__field")
    )

    input_box = element.parent.find_by_css(".select2-search__field")[0]

    list(input_box.type(value, slowly=True))

    wait_for(
        lambda: "Trwa wyszukiwanie…"
        not in browser.find_by_id(f"select2-{element_id}-results").value
    )

    input_box.type(Keys.ENTER)
    time.sleep(0.5)

    if wait_for_new_value:
        try:
            wait_for(
                lambda: browser.find_by_id(f"select2-{element_id}-container").text
                != old_value
            )
        except TimeoutError as e:
            raise e


def select_select2_clear_selection(browser, element_id):
    browser.find_by_id(element_id)[0]
    browser.execute_script(
        "django.jQuery('#" + element_id + "').val(null).trigger('change')"
    )


def select_element_by_text(browser, element_id, text):
    element = browser.find_by_id(element_id)
    show_element(browser, element)
    element.select_by_text(text)


def set_element(browser, element_id, text):
    element = browser.find_by_id(element_id)
    show_element(browser, element)
    element.type(text)


def submit_admin_form(browser):
    with wait_for_page_load(browser):
        browser.execute_script(
            'django.jQuery("input[type=submit].grp-default").click()'
        )


def proper_click_element(browser, element):
    # show_element(browser, element)
    # return element.click()
    browser.execute_script("arguments[0].scrollIntoView();", element._element)
    WebDriverWait(browser, SHORT_WAIT_TIME).until(visibility_of(element._element))
    browser.execute_script("arguments[0].click();", element._element)


def proper_click_by_id(browser, arg):
    elem = browser.find_by_id(arg)
    proper_click_element(browser, elem)


def assertPopupContains(browser, text, accept=True):
    """Switch to popup, assert it contains at least a part
    of the text, close the popup. Error otherwise.
    """
    alert = browser.driver.switch_to.alert
    if text not in alert.text:
        raise AssertionError(f"{text!r} not found in {alert.text!r}")
    if accept:
        alert.accept()


def add_extra_autor_inline(browser, no_current_inlines=0):
    elem = None

    WebDriverWait(browser, SHORT_WAIT_TIME).until(
        lambda browser: not browser.find_by_css(".grp-add-handler").is_empty()
    )

    elems = browser.find_by_css(".grp-add-handler")

    for e in elems:
        if (
            e.visible
            and e.text.find("Dodaj") >= 0
            and e.text.find("powiązanie autora") >= 0
        ):
            elem = e
            break

    proper_click_element(browser, elem)

    try:
        wait_for(
            lambda: not browser.find_by_id(
                f"id_autorzy_set-{no_current_inlines}-autor"
            ).is_empty(),
            max_seconds=LONG_WAIT_TIME,
        )
    except TimeoutError as e:
        raise e


def randomobj(model):
    return model.objects.order_by("?").first()


def quick_find_by_id(browser, id):
    if f'id="{id}"' in browser.html:
        return browser.find_by_id(id, wait_time=0.01)


def fill_admin_form(
    browser,
    zrodlo=None,
    tytul_oryginalny="tytul oryginalny",
    jezyk=None,
    charakter_formalny=None,
    typ_kbn=None,
    status_korekty=None,
    rok=None,
):
    set_element(browser, "id_tytul_oryginalny", tytul_oryginalny)

    if quick_find_by_id(browser, "id_zrodlo"):
        if zrodlo is None:
            # from bpp.models import Zrodlo
            zrodlo = randomobj(Zrodlo)
        select_select2_autocomplete(browser, "id_zrodlo", zrodlo.nazwa)

    if quick_find_by_id(browser, "id_jezyk"):
        if jezyk is None:
            # from bpp.models import Jezk
            jezyk = randomobj(Jezyk)
        select_element_by_text(browser, "id_jezyk", jezyk.nazwa)

    if quick_find_by_id(browser, "id_charakter_formalny"):
        if charakter_formalny is None:
            # charakter_formalny = randomobj(Charakter_Formalny)
            charakter_formalny = Charakter_Formalny.objects.get(nazwa="Broszura")
        select_element_by_text(
            browser, "id_charakter_formalny", " " + charakter_formalny.nazwa
        )

    if quick_find_by_id(browser, "id_typ_kbn"):
        if typ_kbn is None:
            typ_kbn = randomobj(Typ_KBN)
        select_element_by_text(browser, "id_typ_kbn", typ_kbn.nazwa)

    if status_korekty is None:
        status_korekty = randomobj(Status_Korekty)
    select_element_by_text(browser, "id_status_korekty", "przed korektą")

    if rok is None:
        rok = random.randint(1, 2100)
    set_element(browser, "id_rok", rok)


def fill_admin_inline(
    browser,
    autor,
    jednostka,
    zapisany_jako=None,
    procent=None,
    no=0,
    prefix=None,
    dyscyplina=None,
):
    "Daj prefix równy 'id_' aby wypełniać pojedyncze formularze (nie inlines)"

    # Poza tymi wypełnianymi poniżej sa jeszcze:
    # autorzy_set-0-typ_odpowiedzialnosci
    # autorzy_set-0-afiliuje
    # autorzy_set-0-zatrudniony
    # autorzy_set-0-kolejnosc
    # autorzy_set-0-rekord
    # autorzy_set-0-id

    if prefix is None:
        prefix = f"id_autorzy_set-{no}-"

    select_select2_autocomplete(
        browser, f"{prefix}autor", f"{autor.nazwisko} {autor.imiona}"
    )
    select_select2_autocomplete(browser, f"{prefix}jednostka", jednostka.nazwa)
    if zapisany_jako is None:
        zapisany_jako = f"{autor.nazwisko} {autor.imiona}"
    select_select2_autocomplete(browser, f"{prefix}zapisany_jako", zapisany_jako)
    if procent is not None:
        set_element(browser, f"{prefix}procent", procent)
    if procent == -1:
        set_element(browser, f"{prefix}procent", str(random.randint(1, 100)) + ".00")

    if dyscyplina:
        # set_element(browser, f"{prefix}dyscyplina_naukowa", dyscyplina)
        select_select2_autocomplete(
            browser, f"{prefix}dyscyplina_naukowa", dyscyplina.nazwa
        )


def submitted_form_bad(browser, wait_time=SHORT_WAIT_TIME):
    WebDriverWait(browser.driver, wait_time).until(
        lambda driver: "Prosimy poprawić" in driver.page_source
    )
    return True


def submitted_form_good(browser, wait_time=SHORT_WAIT_TIME):
    WebDriverWait(browser.driver, wait_time).until(
        lambda driver: "został dodany pomyślnie" in driver.page_source
    )
    return True


def browse_praca_url(model):
    return reverse(
        "bpp:browse_praca_by_slug", args=(model.slug,)
    )  # ContentType.objects.get_for_model(model).pk, model.pk)


def normalize_html(s: bytes | str, default_encoding="utf-8"):
    if isinstance(s, bytes):
        s = s.decode(default_encoding)
    s = s.replace("\r\n", " ").replace("\n", " ")
    return re.sub(r"\s\s+", " ", s)


def normalize_response_content(res: HttpResponse):
    return normalize_html(
        res.content, cgi.parse_header(res["content-type"])[1]["charset"]
    )


def rozwin_ekstra_informacje_na_stronie_edycji_wydawnictwa(admin_browser):
    for elem in admin_browser.find_by_tag("h2")[:3]:
        show_element(admin_browser, elem)  # ._element)
        elem.click()



# Generated by Django 2.1.10 on 2019-10-21 18:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0183_auto_20191020_1535'),
    ]

    operations = [
        migrations.AddField(
            model_name='autor',
            name='expertus_id',
            field=models.CharField(blank=True, max_length=10, null=True, verbose_name='Identyfikator w bazie Expertus'),
        ),
    ]



# -*- encoding: utf-8 -*-
from django.db.models import F, IntegerField, Max
from django.db.models.functions import Cast

from django.contrib.admin.filters import SimpleListFilter
from django.contrib.admin.models import ADDITION, CHANGE, LogEntry
from django.contrib.contenttypes.models import ContentType

from bpp.models import BppUser, Wydawnictwo_Zwarte
from bpp.models.struktura import Jednostka


class SimpleIntegerFilter(SimpleListFilter):
    db_field_name = None

    def lookups(self, request, model_admin):
        return [
            ("brak", "wartość nie ustalona"),
            ("zero", "zero"),
            ("powyzej", "więcej, niż zero"),
        ]

    def queryset(self, request, queryset):
        v = self.value()

        field = self.db_field_name
        if field is None:
            field = self.parameter_name

        if v == "brak":
            return queryset.filter(**{field: None})
        elif v == "zero":
            return queryset.filter(**{field: 0})
        elif v == "powyzej":
            return queryset.filter(**{field + "__gt": 0})

        return queryset


class SimpleNotNullFilter(SimpleListFilter):
    db_field_name = None

    def lookups(self, request, model_admin):
        return [("brak", "wartość nie ustalona"), ("jest", "wartość ustalona")]

    def queryset(self, request, queryset):
        v = self.value()

        field = self.db_field_name
        if field is None:
            field = self.parameter_name

        if v == "brak":
            return queryset.filter(**{field: None})
        elif v == "jest":
            return queryset.exclude(**{field: None})

        return queryset


class JestWydawnictwemNadrzednymDlaFilter(SimpleNotNullFilter):
    title = "jest wydawnictwem nadrzędnym dla innego"
    parameter_name = "wyd_nad"

    def lookups(self, request, model_admin):
        return [("brak", "nie jest"), ("jest", "jest")]

    def queryset(self, request, queryset):
        v = self.value()
        field = self.db_field_name
        if field is None:
            field = self.parameter_name

        if v == "brak":
            return queryset.exclude(
                pk__in=Wydawnictwo_Zwarte.objects.wydawnictwa_nadrzedne_dla_innych()
            )
        elif v == "jest":
            return queryset.filter(
                pk__in=Wydawnictwo_Zwarte.objects.wydawnictwa_nadrzedne_dla_innych()
            )

        return queryset


class MaWydawnictwoNadrzedneFilter(SimpleNotNullFilter):
    title = "ma wydawnictwo nadrzędne"
    parameter_name = "ma_wyd_nad"

    def lookups(self, request, model_admin):
        return [("brak", "ma"), ("jest", "nie ma")]

    def queryset(self, request, queryset):
        v = self.value()
        field = self.db_field_name
        if field is None:
            field = self.parameter_name

        if v == "brak":
            # ma nadrzedne
            return queryset.exclude(wydawnictwo_nadrzedne_id=None)
        elif v == "jest":
            # nie ma nadrzednego
            return queryset.filter(wydawnictwo_nadrzedne_id=None)

        return queryset


class LiczbaZnakowFilter(SimpleIntegerFilter):
    title = "liczba znaków wydawniczych"
    parameter_name = "liczba_znakow_wydawniczych"


class DOIUstawioneFilter(SimpleNotNullFilter):
    title = "DOI ustawione"
    parameter_name = "doi"


class PBNIDObecnyFilter(SimpleNotNullFilter):
    title = "PBN ID"
    parameter_name = "pbn_id"


class OrcidObecnyFilter(SimpleNotNullFilter):
    title = "ORCID"
    parameter_name = "orcid"


class OrcidAutoraDyscyplinyObecnyFilter(SimpleNotNullFilter):
    title = "ORCID autora"
    parameter_name = "autor__orcid"


class PBN_UID_IDAutoraObecnyFilter(SimpleNotNullFilter):
    title = "PBN UID autora"
    parameter_name = "autor__pbn_uid_id"


class PBN_UID_IDObecnyFilter(SimpleNotNullFilter):
    title = "PBN UID"
    parameter_name = "pbn_uid_id"


class CalkowitaLiczbaAutorowFilter(SimpleIntegerFilter):
    title = "całkowita liczba autorów"
    parameter_name = "calkowita_liczba_autorow"


class JednostkaFilter(SimpleListFilter):
    title = "Jednostka"
    parameter_name = "jednostka"

    def queryset(self, request, queryset):
        v = self.value()
        if v:
            return queryset.filter(aktualna_jednostka_id=v)
        return queryset

    def lookups(self, request, model_admin):
        return (
            (x.pk, str(x)) for x in Jednostka.objects.all().select_related("wydzial")
        )


class LogEntryFilterBase(SimpleListFilter):
    action_flags = [ADDITION, CHANGE]

    def __init__(self, request, params, model, model_admin):
        self.content_type = ContentType.objects.get_for_model(model)
        super(LogEntryFilterBase, self).__init__(
            request=request, params=params, model=model, model_admin=model_admin
        )

    def logentries(self):
        return LogEntry.objects.filter(
            action_flag__in=self.action_flags,
            content_type_id=self.content_type,
        )

    def lookups(self, request, model_admin):
        return (
            (x.pk, x.username)
            for x in BppUser.objects.filter(
                pk__in=self.logentries().values_list("user_id")
            ).only("pk", "username")
        )


class OstatnioZmienionePrzezFilter(LogEntryFilterBase):
    title = "Ostatnio zmieniony przez"
    parameter_name = "ostatnio_zmieniony_przez"

    def queryset(self, request, queryset):
        v = self.value()
        if v:
            res = (
                self.logentries()
                .values("object_id", "content_type_id")
                .annotate(
                    max_action_time=Max("action_time"),
                    max_pk=F("object_id"),
                    max_user=F("user_id"),
                )
                .filter(max_user=v)
                .values_list(Cast("object_id", IntegerField()))
            )

            return queryset.filter(pk__in=res)
        return queryset


class UtworzonePrzezFilter(LogEntryFilterBase):
    title = "Utworzone przez"
    parameter_name = "utworzone_przez"

    action_flags = [ADDITION]

    def queryset(self, request, queryset):
        v = self.value()
        if v:
            res = (
                self.logentries()
                .filter(user_id=v)
                .values_list(Cast("object_id", IntegerField()))
            )

            return queryset.filter(pk__in=res)
        return queryset



# Generated by Django 3.0.7 on 2020-07-27 21:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0216_element_repozytorium"),
    ]

    operations = [
        migrations.CreateModel(
            name="Grant",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("nazwa_projektu", models.TextField(blank=True, null=True)),
                ("zrodlo_finansowania", models.TextField(blank=True, null=True)),
                ("numer_projektu", models.CharField(max_length=200)),
                ("rok", models.PositiveSmallIntegerField()),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-07-07 14:17
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import mptt.fields


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0147_auto_20180707_1517'),
    ]

    operations = [
        migrations.AddField(
            model_name='charakter_formalny',
            name='level',
            field=models.PositiveIntegerField(db_index=True, default=0, editable=False),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='charakter_formalny',
            name='lft',
            field=models.PositiveIntegerField(db_index=True, default=0, editable=False),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='charakter_formalny',
            name='parent',
            field=mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='bpp.Charakter_Formalny'),
        ),
        migrations.AddField(
            model_name='charakter_formalny',
            name='rght',
            field=models.PositiveIntegerField(db_index=True, default=0, editable=False),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='charakter_formalny',
            name='tree_id',
            field=models.PositiveIntegerField(db_index=True, default=0, editable=False),
            preserve_default=False,
        ),
    ]



from django.contrib.contenttypes.admin import GenericStackedInline, GenericTabularInline
from django import forms
from django.contrib import admin
from bpp.models import Grant, Grant_Rekordu


@admin.register(Grant)
class GrantAdmin(admin.ModelAdmin):
    list_display = ["nazwa_projektu", "numer_projektu", "rok", "zrodlo_finansowania"]
    search_fields = ["nazwa_projektu", "numer_projektu", "rok", "zrodlo_finansowania"]


class Grant_RekorduForm(forms.ModelForm):
    class Meta:
        model = Grant_Rekordu
        fields = ["grant"]


class Grant_RekorduInline(GenericTabularInline):
    model = Grant_Rekordu
    extra = 0
    form = Grant_RekorduForm



from django.urls import reverse

from raport_slotow.views.upowaznienie_pbn import RaportEwaluacjaUpowaznienia

from bpp.models import Autor_Dyscyplina


def test_raport_ewaluacja_upowaznienia(
    rekord_slotu,
    admin_client,
    rok,
    autor_jan_kowalski,
    dyscyplina1,
    wydawnictwo_ciagle_z_autorem,
):

    Autor_Dyscyplina.objects.create(
        rok=rok, autor=autor_jan_kowalski, dyscyplina_naukowa=dyscyplina1
    )

    rekord_slotu.delete()
    wydawnictwo_ciagle_z_autorem.punkty_kbn = 500
    wydawnictwo_ciagle_z_autorem.save()

    af = wydawnictwo_ciagle_z_autorem.autorzy_set.first()
    af.afiliuje = False
    af.save()

    url = reverse(
        "raport_slotow:raport-ewaluacja-upowaznienia",
    )

    r = RaportEwaluacjaUpowaznienia()
    r.data = dict(od_roku=rok, do_roku=rok, _export="html")
    assert r.get_queryset().count() == 1

    admin_client.get(
        url,
        data={
            "od_roku": 2017,
            "do_roku": 2021,
            "_export": "xlsx",
        },
    )
